@@ -19,3 +19,10 @@
 #undef HAVE_LIBPNG
 #undef GNOME_ENABLE_DEBUG
 #undef NEED_DECLARATION_GETHOSTNAME
+#undef HAVE_SOCKET
+#undef HAVE_STRUCT_LINGER
+#undef USE_EXT2FSLIB
+#undef USE_NETCODE
+#undef USE_TERMNET
+#undef USE_VFS
+#undef HAVE_ORBIT
