@@ -313,10 +313,9 @@ _XmHTMLReadPNG(TWidget html, ImageBuffer *ib)
 
 	/* by now we already know this is a png */
 	ib->next = 8;
-#if 0
-	/* FIXME:  Upgrade my png instaed of this -- mig */
+
 	png_set_sig_bytes(png_ptr, 8);
-#endif
+
 	/* get png info */
 	png_read_info(png_ptr, info_ptr);
 
