@@ -5,10 +5,10 @@
  *
  * Authors: Miguel de Icaza
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public License
+   as published by the Free Software Foundation; either version 2 of
+   the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,8 +17,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <config.h>
 #include <stdio.h>
@@ -29,7 +28,9 @@
 #include <glib.h>
 #include "libgnomeP.h"
 
-#ifndef HAVE_STRNDUP
+#ifdef HAVE_STRNDUP
+#include <string.h>
+#else
 /* Like strdup, but only copy N chars.  */
 extern char *strndup (const char *s, size_t n);
 #endif
