@@ -1,4 +1,10 @@
+#ifndef __GNOME_HELP_H__
+#define __GNOME_HELP_H__ 1
+
 #include <glib.h>
+#include "gnome-defs.h"
+
+BEGIN_GNOME_DECLS
 
 typedef struct {
     gchar *name;
@@ -19,3 +25,7 @@ void gnome_help_display(void *ignore, GnomeHelpMenuEntry *ref);
 
 /* gnome_help_goto() - Cause a help viewer to display file. */
 void gnome_help_goto(void *ignore, gchar *file);
+
+END_GNOME_DECLS
+
+#endif /* __GNOME_HELP_H__ */
