@@ -9,23 +9,21 @@
 #include <unistd.h>
 #include <sys/stat.h>
 #include <glib.h>
-#include "gnome-defs.h"
-#include "gnome-util.h"
-#include "gnome-i18nP.h"
+#include "libgnomeP.h"
 #include <errno.h>
 
 char *gnome_user_home_dir = 0;
 char *gnome_user_dir = 0;
 char *gnome_user_private_dir = 0;
 char *gnome_app_id = 0;
+char gnome_do_not_create_directories = 0;
 
-void
-gnomelib_init (char *app_id)
+static void
+create_user_gnome_directories (void)
 {
-	gnome_user_home_dir = getenv ("HOME");
-	/* never freed - gnome_config currently uses this, and it's better
-	   to figure it out once than to repeatedly get it */
-	gnome_user_dir = g_concat_dir_and_file (gnome_user_home_dir, ".gnome");
+	if (gnome_do_not_create_directories)
+		return;
+	
 	if(
 	   mkdir(gnome_user_dir, 0700) /* This is per-user info
 					   - no need for others to see it */
@@ -49,6 +47,17 @@ gnomelib_init (char *app_id)
 		g_error("Could not set mode 0700 on private per-user Gnome directory <%s> - aborting\n",
 		        gnome_user_private_dir);
 
+}
+
+void
+gnomelib_init (char *app_id)
+{
+	gnome_user_home_dir = getenv ("HOME");
+	/* never freed - gnome_config currently uses this, and it's better
+	   to figure it out once than to repeatedly get it */
+	gnome_user_dir = g_concat_dir_and_file (gnome_user_home_dir, ".gnome");
+	create_user_gnome_directories ();
+
 	gnome_app_id = app_id;
 
 	gnome_i18n_init ();
