@@ -12,6 +12,7 @@
 #include <gtk/gtk.h>
 #include "libgnome/libgnomeP.h"
 #include "gnome-calculator.h"
+#include "gnome-pixmap.h"
 
 typedef void (*sighandler_t)(int);
 
@@ -46,7 +47,7 @@ static void gnome_calculator_destroy	(GtkObject		*object);
 
 static GtkVBoxClass *parent_class;
 
-static GdkPixmap * font_pixmap=NULL;
+static GtkWidget * font_pixmap=NULL;
 
 typedef struct _CalculatorButton CalculatorButton;
 struct _CalculatorButton {
@@ -175,32 +176,32 @@ put_led_font(GnomeCalculator *gc)
 	gdk_draw_rectangle(p, style->black_gc, 1, 0, 0, -1, -1);
 
 	if(gc->memory!=0) {
-		gdk_draw_pixmap(p, style->white_gc, font_pixmap, 13*FONT_WIDTH,
+		gdk_draw_pixmap(p, style->white_gc, GNOME_PIXMAP(font_pixmap)->pixmap, 13*FONT_WIDTH,
 				0, 0, 0, FONT_WIDTH, FONT_HEIGHT);
 	}
 	for(x=12,i=strlen(text)-1;i>=0;x--,i--) {
 		if(text[i]>='0' && text[i]<='9')
-			gdk_draw_pixmap(p, style->white_gc, font_pixmap,
+			gdk_draw_pixmap(p, style->white_gc, GNOME_PIXMAP(font_pixmap)->pixmap,
 					(text[i]-'0')*FONT_WIDTH, 0, 
 					x*FONT_WIDTH, 0,
 					FONT_WIDTH, FONT_HEIGHT);
 		else if(text[i]=='.')
-			gdk_draw_pixmap(p, style->white_gc, font_pixmap,
+			gdk_draw_pixmap(p, style->white_gc, GNOME_PIXMAP(font_pixmap)->pixmap,
 					10*FONT_WIDTH, 0, 
 					x*FONT_WIDTH, 0,
 					FONT_WIDTH, FONT_HEIGHT);
 		else if(text[i]=='+')
-			gdk_draw_pixmap(p, style->white_gc, font_pixmap,
+			gdk_draw_pixmap(p, style->white_gc, GNOME_PIXMAP(font_pixmap)->pixmap,
 					11*FONT_WIDTH, 0, 
 					x*FONT_WIDTH, 0,
 					FONT_WIDTH, FONT_HEIGHT);
 		else if(text[i]=='-')
-			gdk_draw_pixmap(p, style->white_gc, font_pixmap,
+			gdk_draw_pixmap(p, style->white_gc, GNOME_PIXMAP(font_pixmap)->pixmap,
 					12*FONT_WIDTH, 0, 
 					x*FONT_WIDTH, 0,
 					FONT_WIDTH, FONT_HEIGHT);
 		else if(text[i]=='e')
-			gdk_draw_pixmap(p, style->white_gc, font_pixmap,
+			gdk_draw_pixmap(p, style->white_gc, GNOME_PIXMAP(font_pixmap)->pixmap,
 					14*FONT_WIDTH, 0, 
 					x*FONT_WIDTH, 0,
 					FONT_WIDTH, FONT_HEIGHT);
@@ -1023,18 +1024,14 @@ gnome_calculator_realized(GtkWidget *w, gpointer data)
 	GnomeCalculator *gc = GNOME_CALCULATOR(w);
 
 	if(!font_pixmap) {
-		GdkColor black;
 		gchar *file =
-			gnome_unconditional_pixmap_file("calculator-font.xpm");
+			gnome_unconditional_pixmap_file("calculator-font.png");
 
 		if(!file) {
-			g_warning("Can't find calculator-font.xpm");
+			g_warning("Can't find calculator-font.png");
 			return FALSE;
 		}
-		gdk_color_black(gtk_widget_get_colormap(GTK_WIDGET(gc)),
-							&black);
-		font_pixmap = gdk_pixmap_create_from_xpm(GTK_WIDGET(gc)->window,
-							 NULL, &black, file);
+		font_pixmap = gnome_pixmap_new_from_file(file);
 		g_free(file);
 	}
 
@@ -1120,11 +1117,7 @@ gnome_calculator_init (GnomeCalculator *gc)
 	gc->mode = GNOME_CALCULATOR_DEG;
 	gc->invert = FALSE;
 	gc->add_digit = FALSE;
-#ifdef GTK_HAVE_FEATURES_1_1_0
 	gc->accel = gtk_accel_group_new();
-#else
-	gc->accel = gtk_accelerator_table_new();
-#endif
 
 	gtk_signal_connect_after(GTK_OBJECT(gc),"realize",
 				 GTK_SIGNAL_FUNC(gnome_calculator_realized),
@@ -1143,7 +1136,7 @@ gnome_calculator_init (GnomeCalculator *gc)
 				gtk_signal_connect(GTK_OBJECT(w),"clicked",
 						   but->signal_func,
 						   but);
-#ifdef GTK_HAVE_FEATURES_1_1_0
+
 				if(but->key) {
 				  gtk_widget_add_accelerator(w,
 							     "clicked",
@@ -1163,24 +1156,7 @@ gnome_calculator_init (GnomeCalculator *gc)
 							     GDK_LOCK_MASK,
 							     GTK_ACCEL_VISIBLE);
 				}
-#else
-                                if(but->key) {
-                                        gtk_widget_install_accelerator(w,
-                                                gc->accel,
-                                                "clicked",
-                                                but->key,0);
-                                        gtk_widget_install_accelerator(w,
-                                                gc->accel,
-                                                "clicked",
-                                                but->key,
-                                                GDK_SHIFT_MASK);
-                                        gtk_widget_install_accelerator(w,
-                                                gc->accel,
-                                                "clicked",
-                                                but->key,
-                                                GDK_LOCK_MASK);
-				}
-#endif
+
 				gtk_object_set_user_data(GTK_OBJECT(w),gc);
 				gtk_widget_show(w);
 				gtk_table_attach(GTK_TABLE(table),w,
