@@ -10,30 +10,35 @@
 
 /* TO-DO list for the canvas:
  *
- * - Have the canvas carry a list of areas to repaint.  If a request_repaint is made for an area
- *   that does not touch any of the areas of the list, queue the new area.  Else, grow the existing,
- *   touched area (see Gnumeric for reasons).
+ * - Implement hiding/showing of items (visibility flag).
  *
- * - Implement gnome_canvas_item_reparent().
+ * - Allow to specify whether GnomeCanvasImage sizes are in units or pixels (scale or don't scale).
+ *
+ * - Implement a flag for gnome_canvas_item_reparent() that tells the function to keep the item
+ *   visually in the same place, that is, to keep it in the same place with respect to the canvas
+ *   origin.
  *
  * - GC put functions for items.
  *
  * - Stipple for filling items.
  *
+ * - Widget item (finish it).
+ *
+ * - GList *gnome_canvas_gimme_all_items_contained_in_this_area (GnomeCanvas *canvas, Rectangle area);
+ *
+ * - Have the canvas carry a list of areas to repaint.  If a request_repaint is made for an area
+ *   that does not touch any of the areas of the list, queue the new area.  Else, grow the existing,
+ *   touched area (see Gnumeric for reasons).
+ *
  * - Curve support for line item.
  *
  * - Polygon item.
  *
  * - Arc item.
  *
- * - Widget item.
- *
- * - Text item: wrapped and clipped text, support justification.
- *
  * - Sane font handling API.
  *
  * - Get_arg methods for items:
- *   - How to query colors?
  *   - How to fetch the outline width and know whether it is in pixels or units?
  *
  * - Talk Raph into using a very similar API for GtkCaanvas.
@@ -42,9 +47,8 @@
  *
  * - item_class->output (item, "bridge_to_raph's_canvas")
  *
- * - Multiple exposure event compression; this may need to be in Gtk/Gdk instead.
- */
-
+ * - Multiple exposure event compression; this may need to be in Gtk/Gdk instead.  */
+#include <stdio.h>
 #include <config.h>
 #include <math.h>
 #include <gdk/gdkprivate.h>
@@ -145,7 +149,7 @@ gnome_canvas_item_class_init (GnomeCanvasItemClass *class)
 }
 
 GnomeCanvasItem *
-gnome_canvas_item_new (GnomeCanvasGroup *parent, GtkType type, ...)
+gnome_canvas_item_new (GnomeCanvasGroup *parent, GtkType type, const gchar *first_arg_name, ...)
 {
 	GnomeCanvasItem *item;
 	va_list args;
@@ -154,10 +158,10 @@ gnome_canvas_item_new (GnomeCanvasGroup *parent, GtkType type, ...)
 	g_return_val_if_fail (GNOME_IS_CANVAS_GROUP (parent), NULL);
 	g_return_val_if_fail (gtk_type_is_a (type, gnome_canvas_item_get_type ()), NULL);
 
-	item = GNOME_CANVAS_ITEM(gtk_type_new (type));
+	item = GNOME_CANVAS_ITEM (gtk_type_new (type));
 
-	va_start (args, type);
-	gnome_canvas_item_construct (item, parent, args);
+	va_start (args, first_arg_name);
+	gnome_canvas_item_construct (item, parent, first_arg_name, args);
 	va_end (args);
 
 	return item;
@@ -193,7 +197,7 @@ item_post_create_setup (GnomeCanvasItem *item)
 }
 
 void
-gnome_canvas_item_construct (GnomeCanvasItem *item, GnomeCanvasGroup *parent, va_list args)
+gnome_canvas_item_construct (GnomeCanvasItem *item, GnomeCanvasGroup *parent, const gchar *first_arg_name, va_list args)
 {
         GtkObject *obj;
 	GSList *arg_list;
@@ -211,7 +215,7 @@ gnome_canvas_item_construct (GnomeCanvasItem *item, GnomeCanvasGroup *parent, va
 	arg_list = NULL;
 	info_list = NULL;
 
-	error = gtk_object_args_collect (GTK_OBJECT_TYPE (obj), &arg_list, &info_list, &args);
+	error = gtk_object_args_collect (GTK_OBJECT_TYPE (obj), &arg_list, &info_list, first_arg_name, args);
 
 	if (error) {
 		g_warning ("gnome_canvas_item_construct(): %s", error);
@@ -332,9 +336,18 @@ gnome_canvas_item_marshal_signal_1 (GtkObject *object, GtkSignalFunc func, gpoin
 }
 
 void
-gnome_canvas_item_set (GnomeCanvasItem *item, ...)
+gnome_canvas_item_set (GnomeCanvasItem *item, const gchar *first_arg_name, ...)
 {
 	va_list args;
+
+	va_start (args, first_arg_name);
+	gnome_canvas_item_set_valist (item, first_arg_name, args);
+	va_end (args);
+}
+
+void 
+gnome_canvas_item_set_valist (GnomeCanvasItem *item, const gchar *first_arg_name, va_list args)
+{
 	GSList *arg_list;
 	GSList *info_list;
 	char *error;
@@ -345,9 +358,7 @@ gnome_canvas_item_set (GnomeCanvasItem *item, ...)
 	arg_list = NULL;
 	info_list = NULL;
 
-	va_start (args, item);
-	error = gtk_object_args_collect (GTK_OBJECT_TYPE (item), &arg_list, &info_list, &args);
-	va_end (args);
+	error = gtk_object_args_collect (GTK_OBJECT_TYPE (item), &arg_list, &info_list, first_arg_name, args);
 
 	if (error) {
 		g_warning ("gnome_canvas_item_set(): %s", error);
@@ -371,6 +382,7 @@ gnome_canvas_item_set (GnomeCanvasItem *item, ...)
 	}
 }
 
+
 void
 gnome_canvas_item_setv (GnomeCanvasItem *item, guint nargs, GtkArg *args)
 {
@@ -636,6 +648,64 @@ gnome_canvas_item_i2w (GnomeCanvasItem *item, double *x, double *y)
 	}
 }
 
+/* Returns whether the item is an inferior of or is equal to the parent. */
+static int
+is_descendant (GnomeCanvasItem *item, GnomeCanvasItem *parent)
+{
+	for (; item; item = item->parent)
+		if (item == parent)
+			return TRUE;
+
+	return FALSE;
+}
+
+void
+gnome_canvas_item_reparent (GnomeCanvasItem *item, GnomeCanvasGroup *new_group)
+{
+	GnomeCanvasItem *old_parent;
+
+	g_return_if_fail (item != NULL);
+	g_return_if_fail (GNOME_IS_CANVAS_ITEM (item));
+	g_return_if_fail (new_group != NULL);
+	g_return_if_fail (GNOME_IS_CANVAS_GROUP (new_group));
+
+	/* Both items need to be in the same canvas */
+
+	g_return_if_fail (item->canvas == GNOME_CANVAS_ITEM (new_group)->canvas);
+
+	/* The group cannot be an inferior of the item or be the item itself -- this also takes care
+	 * of the case where the item is the root item of the canvas.
+	 */
+
+	g_return_if_fail (!is_descendant (GNOME_CANVAS_ITEM (new_group), item));
+
+	/* Everything is ok, now actually reparent the item */
+
+	gtk_object_ref (GTK_OBJECT (item)); /* protect it from the unref in group_remove */
+
+	gnome_canvas_request_redraw (item->canvas, item->x1, item->y1, item->x2, item->y2);
+
+	if (item->parent)
+		group_remove (GNOME_CANVAS_GROUP (item->parent), item);
+
+	item->parent = GNOME_CANVAS_ITEM (new_group);
+	group_add (new_group, item);
+
+	/* Rebuild the new parent group's bounds.  This will take care of reconfiguring the item and
+	 * all its children.
+	 */
+
+	gnome_canvas_group_child_bounds (new_group, NULL);
+
+	/* Redraw and repick */
+
+	gnome_canvas_request_redraw (item->canvas, item->x1, item->y1, item->x2, item->y2);
+	item->canvas->need_repick = TRUE;
+
+	gtk_object_unref (GTK_OBJECT (item));
+}
+
+  
 
 /*** GnomeCanvasGroup ***/
 
@@ -1487,7 +1557,7 @@ scroll_to (GnomeCanvas *canvas, int cx, int cy)
 	if (changed)
 		gtk_layout_freeze (GTK_LAYOUT (canvas));
 
-	if ((scroll_maxx != canvas->layout.width) || (scroll_maxy != canvas->layout.height))
+	if ((scroll_maxx != (int) canvas->layout.width) || (scroll_maxy != (int) canvas->layout.height))
 		gtk_layout_set_size (GTK_LAYOUT (canvas), scroll_maxx, scroll_maxy);
 
 	if (changed_x) {
@@ -1541,16 +1611,6 @@ window_to_world (GnomeCanvas *canvas, double *x, double *y)
 	*y = canvas->scroll_y1 + (*y + DISPLAY_Y1 (canvas) - canvas->zoom_yofs) / canvas->pixels_per_unit;
 }
 
-static int
-is_descendant (GnomeCanvasItem *item, GnomeCanvasItem *parent)
-{
-	for (; item; item = item->parent)
-		if (item == parent)
-			return TRUE;
-
-	return FALSE;
-}
-
 static void
 emit_event (GnomeCanvas *canvas, GdkEvent *event)
 {
@@ -1634,11 +1694,12 @@ emit_event (GnomeCanvas *canvas, GdkEvent *event)
 	 * a leaf event), and emission is stopped if a handler returns TRUE, just like for GtkWidget
 	 * events.
 	 */
-
 	for (finished = FALSE, item = canvas->current_item; item && !finished; item = item->parent)
+	  {
 		gtk_signal_emit (GTK_OBJECT (item), item_signals[ITEM_EVENT],
 				 &ev,
 				 &finished);
+	  }
 }
 
 static void
@@ -2033,13 +2094,13 @@ idle_handler (gpointer data)
 	return FALSE;
 }
 
-GnomeCanvasItem *
+GnomeCanvasGroup *
 gnome_canvas_root (GnomeCanvas *canvas)
 {
 	g_return_val_if_fail (canvas != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_CANVAS (canvas), NULL);
 
-	return canvas->root;
+	return GNOME_CANVAS_GROUP (canvas->root);
 }
 
 void
