@@ -119,23 +119,27 @@ GtkType gnome_canvas_item_get_type (void);
 /* Create a canvas item using the standard Gtk argument mechanism.  The item is automatically
  * inserted at the top of the specified canvas group.  The last argument must be a NULL pointer.
  */
-GnomeCanvasItem *gnome_canvas_item_new (GnomeCanvasGroup *parent, GtkType type, ...);
+GnomeCanvasItem *gnome_canvas_item_new (GnomeCanvasGroup *parent, GtkType type, const gchar *first_arg_name, ...);
 
 /* Same as above, with parsed args */
 GnomeCanvasItem *gnome_canvas_item_newv (GnomeCanvasGroup *parent, GtkType type, guint nargs, GtkArg *args);
 
 /* Constructors for use in derived classes and language wrappers */
-void gnome_canvas_item_construct (GnomeCanvasItem *item, GnomeCanvasGroup *parent, va_list args);
+void gnome_canvas_item_construct (GnomeCanvasItem *item, GnomeCanvasGroup *parent, const gchar *first_arg_name, va_list args);
 
 void gnome_canvas_item_constructv (GnomeCanvasItem *item, GnomeCanvasGroup *parent,
 				   guint nargs, GtkArg *args);
 
 /* Configure an item using the standard Gtk argument mechanism.  The last argument must be a NULL pointer. */
-void gnome_canvas_item_set (GnomeCanvasItem *item, ...);
+void gnome_canvas_item_set (GnomeCanvasItem *item, const gchar *first_arg_name, ...);
 
 /* Same as above, with parsed args */
 void gnome_canvas_item_setv (GnomeCanvasItem *item, guint nargs, GtkArg *args);
 
+/* Used only for language wrappers and the like; ignore. */
+void gnome_canvas_item_set_valist (GnomeCanvasItem *item, const gchar *first_arg_name,
+				   va_list var_args);
+
 /* Move an item by the specified amount */
 void gnome_canvas_item_move (GnomeCanvasItem *item, double dx, double dy);
 
@@ -174,6 +178,15 @@ void gnome_canvas_item_ungrab (GnomeCanvasItem *item, guint32 etime);
 void gnome_canvas_item_w2i (GnomeCanvasItem *item, double *x, double *y);
 void gnome_canvas_item_i2w (GnomeCanvasItem *item, double *x, double *y);
 
+/* Remove the item from its parent group and make the new group its parent.  The item will be put on
+ * top of all the items in the new group.  The item's coordinates relative to its new parent to
+ * *not* change -- this means that the item could potentially move on the screen.
+ * 
+ * The item and the group must be in the same canvas.  An item cannot be reparented to a group that
+ * is the item itself or that is an inferior of the item.
+ */
+void gnome_canvas_item_reparent (GnomeCanvasItem *item, GnomeCanvasGroup *new_group);
+
 
 /* GnomeCanvasGroup - a group of canvas items
  *
@@ -305,7 +318,7 @@ GtkType gnome_canvas_get_type (void);
 GtkWidget *gnome_canvas_new (void);
 
 /* Returns the root canvas item group of the canvas */
-GnomeCanvasItem *gnome_canvas_root (GnomeCanvas *canvas);
+GnomeCanvasGroup *gnome_canvas_root (GnomeCanvas *canvas);
 
 /* Sets the limits of the scrolling region */
 void gnome_canvas_set_scroll_region (GnomeCanvas *canvas, double x1, double y1, double x2, double y2);
