@@ -106,6 +106,9 @@ struct _GnomeClient
   /* Prefix for configuration files.  */
   gchar              *global_config_prefix;
 
+  /* Static command line options.  */
+  GList              *static_args;
+
   /* The following properties are predefined in the X session
      management protocol.  The entries marked with a 'x' are required
      by the session management protocol.  The entries marked with a
@@ -113,7 +116,7 @@ struct _GnomeClient
   gchar             **clone_command;        /*[xs]*/
   gchar              *current_directory;    /*[  ]*/
   gchar             **discard_command;      /*[  ]*/
-  gchar             **environment;          /*[  ]*/
+  GList              *environment;          /*[  ]*/
   pid_t               process_id;           /*[ s]*/
   gchar              *program;              /*[xs]*/
   gchar             **resign_command;       /*[  ]*/
@@ -161,18 +164,49 @@ struct _GnomeClientClass
 
 guint        gnome_client_get_type (void);
 
-/* Create the default session management client.  This registers some
-   command-line options with gnome_parse_register_arguments.  If
-   possible the default client will contact the session manager after
-   command-line parsing is finished.  The default client will also set
-   the SM_CLIENT_ID property on the main window of your application.  */
+
+/* Obsolet, use 'gnome_master_client' instead.  This function will be
+   removed soon.  */
 GnomeClient *gnome_client_new_default	         (void);
 
+
+/* Normally the master client is connected to the session manager
+   automatically, when calling 'gnome_init'.  One can disable this
+   automatic connect by calling this function. Using this function
+   should definitely be an exception.  */
+void         gnome_client_disable_master_connection (void);
+
+/* Get the master session management client.  This master client gets
+   a client id, that may be specified by the '--sm-client-id' command
+   line option.  A master client will be generated by 'gnome-init'.
+   If possible the master client will contact the session manager
+   after command-line parsing is finished (except
+   'gnome_client_disable_master_connection' was called).  The master
+   client will also set the SM_CLIENT_ID property on the main window
+   of your application.  
+
+   Additional the master client gets some static arguments set
+   automatically (see 'gnome_client_add_static_arg' for static
+   arguments): 'gnome_init' sets all command line options, that are
+   understood be 'gnome_init' as static arguments to the master
+   client. */
+GnomeClient *gnome_master_client 	         (void);
+
+/* Get the cloned session management client.  This client gets a
+   client id, that may be specified by the '--sm-cloned-id' command
+   line option.  If no '--sm-cloned-id' command line was given, the
+   '--sm-client-id' option will be evaluated.  This client is
+   intentend to be used in combination with the
+   'gnome_client_get_[global_]config_prefix' commands, in order to get
+   the configuration of a cloned client.  This client must not be
+   connected to a session-manager.*/
+GnomeClient *gnome_cloned_client 	         (void);
+
+
 /* Create a new session management client and try to connect to a
    session manager.  */
 GnomeClient *gnome_client_new                    (void);
 
-
 /* Create a new session management client.  */
 GnomeClient *gnome_client_new_without_connection (void);
 
@@ -244,7 +278,8 @@ void         gnome_client_set_current_directory  (GnomeClient *client,
 void         gnome_client_set_discard_command    (GnomeClient *client,
 						  gint argc, gchar *argv[]);
 void         gnome_client_set_environment        (GnomeClient *client,
-						  gint argc, gchar *argv[]);
+						  const gchar *name,
+						  const gchar *value);
 void         gnome_client_set_process_id         (GnomeClient *client, 
 						  pid_t pid);
 void         gnome_client_set_program            (GnomeClient *client, 
@@ -260,6 +295,14 @@ void         gnome_client_set_shutdown_command   (GnomeClient *client,
 void         gnome_client_set_user_id            (GnomeClient *client,
 						  const gchar *user_id);
 
+/* The following function may be used, to add static arguments to the
+   clone and restart command.  This function can be called more than
+   once.  Every call appends the new arguments to the older ones.
+   These arguments are inserted ahead the arguments set with the
+   'gnome_client_set[clone|restart]_command'.  This list of arguments,
+   given to this function must be end with 'NULL'.  */
+void         gnome_client_add_static_arg (GnomeClient *client, ...);
+
 
 /* The following function can be used, if you want that the
    'save_yourself' signal is emitted again, after all clients finished
