@@ -178,15 +178,9 @@ gnome_dialog_init (GnomeDialog *dialog)
   gtk_window_position(GTK_WINDOW(dialog), 
 		      gnome_preferences_get_dialog_position());
   
-#ifdef GTK_HAVE_FEATURES_1_1_0
   dialog->accelerators = gtk_accel_group_new();
   gtk_window_add_accel_group (GTK_WINDOW(dialog), 
 				    dialog->accelerators);
-#else
-  dialog->accelerators = gtk_accelerator_table_new();
-  gtk_window_add_accelerator_table (GTK_WINDOW(dialog),
-                                    dialog->accelerators);
-#endif
 
   bf = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (bf), GTK_SHADOW_OUT);
@@ -463,12 +457,13 @@ gnome_dialog_run_modal(GnomeDialog *dialog)
 gint
 gnome_dialog_run(GnomeDialog *dialog)
 {
-  gboolean was_modal;
+  gboolean was_modal, was_only_hidden;
   struct GnomeDialogRunInfo ri = {-1,-1,-1};
 
   g_return_val_if_fail(dialog != NULL, -1);
   g_return_val_if_fail(GNOME_IS_DIALOG(dialog), -1);
 
+  was_only_hidden = dialog->just_hide || !dialog->click_closes;
   was_modal = dialog->modal;
   if (!was_modal) gnome_dialog_set_modal(dialog);
 
@@ -487,19 +482,21 @@ gnome_dialog_run(GnomeDialog *dialog)
 
   gtk_main();
 
-  if(!was_modal)
-    {
-      /* It would be nicer if gnome_dialog_set_modal() took a
-	 bool value to indicate yes/no modal */
-      dialog->modal = FALSE;
-      gtk_grab_remove(GTK_WIDGET(dialog));
-    }
-
-  if(ri.close_id >= 0)
-    {
-      gtk_signal_disconnect(GTK_OBJECT(dialog), ri.close_id);
-      gtk_signal_disconnect(GTK_OBJECT(dialog), ri.clicked_id);
-    }
+  if(was_only_hidden) {
+    if(!was_modal)
+      {
+	/* It would be nicer if gnome_dialog_set_modal() took a
+	   bool value to indicate yes/no modal */
+	dialog->modal = FALSE;
+	gtk_grab_remove(GTK_WIDGET(dialog));
+      }
+    
+    if(ri.close_id >= 0)
+      {
+	gtk_signal_disconnect(GTK_OBJECT(dialog), ri.close_id);
+	gtk_signal_disconnect(GTK_OBJECT(dialog), ri.clicked_id);
+      }
+  }
 
   return ri.button_number;
 }
@@ -524,15 +521,20 @@ gint
 gnome_dialog_run_and_destroy (GnomeDialog * dialog)
 {
   gint r;
+  gboolean need_close;
 
   g_return_val_if_fail(dialog != NULL, -1);
   g_return_val_if_fail(GNOME_IS_DIALOG(dialog), -1);  
 
+  need_close = dialog->click_closes;
+
   gnome_dialog_close_hides(dialog, FALSE);
 
   r = gnome_dialog_run(dialog);
 
-  gnome_dialog_close(dialog); /* Causes a destroy. */
+  if(need_close)
+    gnome_dialog_close(dialog); /* Causes a destroy. */
+
   return r;
 }
 
@@ -664,22 +666,13 @@ void       gnome_dialog_set_accelerator(GnomeDialog * dialog,
   list = g_list_nth (dialog->buttons, button);
 
   if (list && list->data) {
-#ifdef GTK_HAVE_FEATURES_1_1_0
-/*FIXME*/
+    /*FIXME*/
     gtk_widget_add_accelerator(GTK_WIDGET(list->data),
 			       "clicked",
 			       dialog->accelerators,
 			       accelerator_key,
 			       accelerator_mods,
 			       GTK_ACCEL_VISIBLE);
-#else
-    gtk_widget_install_accelerator(GTK_WIDGET(list->data),
-                                   dialog->accelerators,
-                                   "clicked",
-                                   accelerator_key,
-                                   accelerator_mods);
-
-#endif
     
     return;
   }
@@ -769,11 +762,6 @@ static void gnome_dialog_destroy (GtkObject *dialog)
 
   g_list_free(GNOME_DIALOG (dialog)->buttons);
 
-#ifndef GTK_HAVE_FEATURES_1_1_0
-  if (GNOME_DIALOG(dialog)->accelerators)
-    gtk_accelerator_table_unref(GNOME_DIALOG(dialog)->accelerators);
-#endif
-
   if (GTK_OBJECT_CLASS(parent_class)->destroy)
     (* (GTK_OBJECT_CLASS(parent_class)->destroy))(dialog);
 }
@@ -825,6 +813,82 @@ static void gnome_dialog_show (GtkWidget * d)
 
 /****************************************************************
   $Log: gnome-dialog.c,v $
+  Revision 1.39  1998/09/18 23:25:39  unammx
+  Popup menus work.  The API is very simple, and I think it is enough
+  for most purposes -- if language binding writers think there is
+  additional stuff that could be done to them, please feel free to add
+  it.
+
+  I am writing some DocBook documentation for GnomeAppHelper and popup
+  menus; hopefully it will be finished during the weekend.  People
+  should add underlined letters and accelerators to their menus as soon
+  as possible -- the arrangement of the menus and accelerators still
+  needs to be settled in the UI Guidelines, but that's another story :-)
+
+    Federico
+
+  1998-09-18  Federico Mena Quintero  <federico@nuclecu.unam.mx>
+
+  	* configure.in: Removed generation of list of canvas files.
+
+  1998-09-18  Federico Mena Quintero  <federico@nuclecu.unam.mx>
+
+  	* gnome-popup-menu.c (gnome_popup_menu_new): New public function
+  	to create a popup menu from a GnomeUIInfo array.
+  	(gnome_popup_menu_attach): New public function to attach a popup
+  	menu to a widget -- the popup menu can be called by pressing mouse
+  	button 3 over the widget.  The menu can be shared between multiple
+  	widgets and it will be destroyed when the last widget it is
+  	attached to is destroyed.
+  	(gnome_popup_menu_do_popup): New public function to explicitly
+  	popup a menu.
+
+  	* gnome-app-helper.c (do_ui_signal_connect): Use
+  	gtk_signal_connect_full() instead of gtk_signal_connect_interp(),
+  	since the latter is obsolete (but works anyway...).
+
+  	* stock_demo.c:
+  	* gnome-stock.c:
+  	* gnome-number-entry.c:
+  	* gnome-init.c:
+  	* gnome-dialog.h:
+  	* gnome-dialog.c:
+  	* gnome-dentry-edit.c:
+  	* gnome-client.c (gnome_client_init):
+  	* gnome-calculator.c:
+  	* gnome-app.c:
+  	* libgnomeui.h: Removed occurrences of #ifdef GTK_HAVE_FEATURES_1_1_0.
+  	We require it, anyways.
+
+  	* Makefile.am: Moved all the conditional canvas files into the
+  	main list of sources -- they are always compiled now.
+
+  1998-09-18  Federico Mena Quintero  <federico@nuclecu.unam.mx>
+
+  	* testgnome.c (create_app_helper): Added tests for popup menus.
+
+  	* canvas-fifteen.c:
+  	* canvas-arrowhead.c:
+  	* canvas-primitives.c: Removed occurrences of #ifdef GTK_HAVE_FEATURES_1_1_0.
+
+  Revision 1.38  1998/08/16 04:11:11  sopwith
+
+
+  libgnomeui:
+  	gnome-dialog.c: gnome_dialog_run_and_destroy() doesn't try to
+  			destroy the dialog if it already was destroyed by
+  			the click. (Havoc, can you review please?)
+  	gnome-preferences.c: Remove memory leaks. Some still
+  		             supposedly exist in the
+  			     gnome_config_get_boolean() calls, but I
+  			     couldn't see them.
+
+  			     Please try to do smart things with return
+  			     values :-)
+  	gnome.defs, gnometypebuiltins_evals.c:
+  		I didn't make these changes, but these files have diffs.
+  		Did someone put autogenerated files into CVS?
+
   Revision 1.37  1998/08/05 15:17:09  owen
   Wed Aug  5 11:19:56 1998  Owen Taylor  <otaylor@redhat.com>
 
