@@ -22,9 +22,7 @@
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkeditable.h>
 #include <gtk/gtkfeatures.h>
-#ifdef GTK_HAVE_FEATURES_1_1_0
 #include <gtk/gtkaccelgroup.h>
-#endif
 #include <libgnome/gnome-defs.h>
 
 #include <stdarg.h>
@@ -48,11 +46,8 @@ struct _GnomeDialog
   GtkWidget * action_area; /* A button box, not an hbox */
 
   GList *buttons;
-#ifdef GTK_HAVE_FEATURES_1_1_0
   GtkAccelGroup * accelerators;
-#else 
-  GtkAcceleratorTable *accelerators;
-#endif
+
   int modal : 1;
   int click_closes : 1;
   int just_hide : 1;
