@@ -45,19 +45,22 @@ static GdkPixmap *imlib_image_loader(GdkWindow   *window,
  */
 static struct argp_option our_gtk_options[] =
 {
-	{ N_("\nGtk toolkit options"), -1, NULL, OPTION_DOC,   NULL, -3 },
-	{ "gdk-debug",         -1, NULL,          OPTION_HIDDEN,    NULL, -2 },
-	{ "gdk-no-debug",      -2, NULL,          OPTION_HIDDEN, NULL, 0 },
+	{ N_("\nGtk+ toolkit options"), -1, NULL, OPTION_DOC,   NULL, -3 },
+	{ "gdk-debug",         -1, N_("FLAGS"),   0,      N_("Gdk debuging flags to set"), 0 },
+	{ "gdk-no-debug",      -2, N_("FLAGS"),   0,      N_("Gdk debuging flags to unset"), 0 },
 	{ "display",           -3, N_("DISPLAY"), 0,	  N_("X display to use"), 0 },
-	{ "sync",              -4, NULL,          OPTION_HIDDEN, 	  NULL, 0 },
+	{ "sync",              -4, NULL,          OPTION_HIDDEN, NULL, 0 },
 	{ "no-xshm",           -5, NULL,          0,	  N_("Don't use X shared memory extension"), 0 },
-	{ "name",              -6, N_("NAME"),    0,	  N_("FIXME"), 0 },
-	{ "class",             -7, N_("CLASS"),   0,	  N_("FIXME"), 0 },
+	{ "name",              -6, N_("NAME"),    0,	  N_("Program name as used by the window manager"), 0 },
+	{ "class",             -7, N_("CLASS"),   0,	  N_("Program class as used by the window manager"), 0 },
 	{ "gxid_host",         -8, N_("HOST"),    0,	  N_("FIXME"), 0 },
 	{ "gxid_port",         -9, N_("PORT"),    0,	  N_("FIXME"), 0 },
 	{ "xim-preedit",      -10, N_("STYLE"),   0,	  N_("FIXME"), 0 },
 	{ "xim-status",       -11, N_("STYLE"),   0,	  N_("FIXME"), 0 },
-	{ "g-fatal-warnings", -12, NULL,          OPTION_HIDDEN, NULL, 0 },
+	{ "gtk-debug",        -12, N_("FLAGS"),   0,      N_("Gtk+ debuging flags to set"), 0 },
+	{ "gtk-no-debug",     -13, N_("FLAGS"),   0,      N_("Gtk+ debuging flags to unset"), 0 },
+	{ "g-fatal-warnings", -14, NULL,          0,      N_("Make all warnings fatal"), 0 },
+	{ "gtk-module",       -15, N_("MODULE"),  0,      N_("Load additional Gtk modules"), 0 },
 	{ NULL, 0, NULL, 0, NULL, 0 }
 };
 
@@ -86,13 +89,12 @@ our_gtk_parse_func (int key, char *arg, struct argp_state *state)
 		 * client.
 		 */
 		our_argv[our_argc] = g_strconcat ("--",
-						    our_gtk_options[- key - 1].name,
+						    our_gtk_options[- key].name,
 						    NULL);
 		gnome_client_add_static_arg (client, our_argv [our_argc++], NULL);
 		
 		if (arg)
 		  {
-		    
 		    our_argv[our_argc] = g_strdup (arg);
 		    gnome_client_add_static_arg (client, our_argv[our_argc++], NULL);
 		  }
@@ -113,40 +115,32 @@ our_gtk_parse_func (int key, char *arg, struct argp_state *state)
 		 */
 		client= gnome_master_client ();
 	}
-	else if (key == ARGP_KEY_SUCCESS)
-		return 0;
+	else if (key == ARGP_KEY_SUCCESS || key == ARGP_KEY_ERROR)
+	{
+		int i, copy_ac = our_argc;
+		char **copy = (char **) g_malloc (our_argc * sizeof (char *));
+
+		memcpy (copy, our_argv, our_argc * sizeof (char *));
+		our_argv[our_argc] = NULL;
+
+		gtk_init (&our_argc, &our_argv);
+		gdk_imlib_init ();
+		gnome_type_init();
+
+		gtk_rc_set_image_loader(imlib_image_loader);
+		gnome_rc_parse (program_invocation_name);
+
+		for (i = 0; i < copy_ac; ++i)
+			g_free (copy[i]);
+		g_free (copy);
+		g_free (our_argv); our_argv = NULL;
+	}
 	else
 		return ARGP_ERR_UNKNOWN;
 
 	return 0;
 }
 
-/* Initialize gnome-related libraries, after argument parsing. */
-static void
-gnome_libs_init (void)
-{
-	int i, copy_ac = our_argc;
-	char **copy = (char **) g_malloc (our_argc * sizeof (char *));
-
-	memcpy (copy, our_argv, our_argc * sizeof (char *));
-	our_argv[our_argc] = NULL;
-
-	gtk_init (&our_argc, &our_argv);
-	gdk_imlib_init ();
-	gnome_type_init();
-		
-#ifdef GTK_HAVE_FEATURES_1_1_0
-	/* New feature in 1.1 tree */
-	gtk_rc_set_image_loader(imlib_image_loader);
-#endif
-	gnome_rc_parse (program_invocation_name);
-
-	for (i = 0; i < copy_ac; ++i)
-		g_free (copy[i]);
-	g_free (copy);
-	g_free (our_argv); our_argv = NULL;
-}
-
 static struct argp our_gtk_parser =
 {
 	our_gtk_options,		/* Options.  */
@@ -232,9 +226,10 @@ atexit_handler(void)
 }
 
 error_t
-gnome_init (char *app_id, struct argp *app_args,
-	    int argc, char **argv,
-	    unsigned int flags, int *arg_index)
+gnome_init_with_data (char *app_id, struct argp *app_args,
+		      int argc, char **argv,
+		      unsigned int flags, int *arg_index,
+		      void *user_data)
 {
 	error_t retval;
 
@@ -278,11 +273,9 @@ gnome_init (char *app_id, struct argp *app_args,
 		argp_program_version_hook = default_version_func;
 
 	/* Now parse command-line arguments.  */
-	retval = gnome_parse_arguments (app_args, argc, argv, flags, arg_index);
-
-	/* Initialize related libraries.  */
-	gnome_libs_init ();
-
+	retval = gnome_parse_arguments_with_data (app_args, argc, argv,
+						  flags, arg_index, user_data);
+	
 	/*now set up the handeling of automatic config syncing*/
 	gnome_config_set_set_handler(set_handler,NULL);
 	gnome_config_set_sync_handler(sync_handler,NULL);
@@ -296,6 +289,15 @@ gnome_init (char *app_id, struct argp *app_args,
 	return retval;
 }
 
+error_t
+gnome_init (char *app_id, struct argp *app_args,
+	    int argc, char **argv,
+	    unsigned int flags, int *arg_index)
+{
+	return gnome_init_with_data (app_id, app_args, argc, argv,
+				     flags, arg_index, NULL);
+}
+
 /* perhaps this belongs in libgnome.. move it if you like. */
 
 /* automagically parse all the gtkrc files for us.
