@@ -32,14 +32,14 @@
 static void             gnome_mdi_class_init     (GnomeMDIClass  *);
 static void             gnome_mdi_init           (GnomeMDI *);
 static void             gnome_mdi_destroy        (GtkObject *);
-static void             gnome_mdi_set_active_view(GnomeMDI *, GtkWidget *);
+static void             set_active_view          (GnomeMDI *, GtkWidget *);
 
 static GtkWidget       *find_item_by_label       (GtkMenuShell *, gchar *);
 
 static void             child_list_menu_create     (GnomeMDI *, GnomeApp *);
-static void             child_list_menu_remove_item(GnomeMDI *, GnomeMDIChild *);
-static void             child_list_menu_add_item   (GnomeMDI *, GnomeMDIChild *);
 static void             child_list_activated_cb    (GtkWidget *, GnomeMDI *);
+void                    child_list_menu_remove_item(GnomeMDI *, GnomeMDIChild *);
+void                    child_list_menu_add_item   (GnomeMDI *, GnomeMDIChild *);
 
 static void             app_create               (GnomeMDI *);
 static void             app_destroy              (GnomeApp *);
@@ -90,6 +90,7 @@ enum {
   ADD_VIEW,
   REMOVE_VIEW,
   CHILD_CHANGED,
+  VIEW_CHANGED,
   APP_CREATED,       /* so new GnomeApps can be further customized by applications */
   LAST_SIGNAL
 };
@@ -172,49 +173,55 @@ static void gnome_mdi_class_init (GnomeMDIClass *class) {
 					      object_class->type,
 					      GTK_SIGNAL_OFFSET (GnomeMDIClass, create_menus),
 					      gnome_mdi_marshal_1,
-					      GTK_TYPE_POINTER, 1, GTK_TYPE_POINTER);
+					      gtk_widget_get_type(), 1, gnome_app_get_type());
   mdi_signals[CREATE_TOOLBAR] = gtk_signal_new ("create_toolbar",
 						GTK_RUN_LAST,
 						object_class->type,
 						GTK_SIGNAL_OFFSET (GnomeMDIClass, create_toolbar),
 						gnome_mdi_marshal_1,
-						GTK_TYPE_POINTER, 1, GTK_TYPE_POINTER);
+						gtk_widget_get_type(), 1, gnome_app_get_type());
   mdi_signals[ADD_CHILD] = gtk_signal_new ("add_child",
 					   GTK_RUN_LAST,
 					   object_class->type,
 					   GTK_SIGNAL_OFFSET (GnomeMDIClass, add_child),
 					   gnome_mdi_marshal_2,
-					   GTK_TYPE_BOOL, 1, GTK_TYPE_POINTER);
+					   GTK_TYPE_BOOL, 1, gnome_mdi_child_get_type());
   mdi_signals[REMOVE_CHILD] = gtk_signal_new ("remove_child",
 					      GTK_RUN_LAST,
 					      object_class->type,
 					      GTK_SIGNAL_OFFSET (GnomeMDIClass, remove_child),
 					      gnome_mdi_marshal_2,
-					      GTK_TYPE_BOOL, 1, GTK_TYPE_POINTER);
+					      GTK_TYPE_BOOL, 1, gnome_mdi_child_get_type());
   mdi_signals[ADD_VIEW] = gtk_signal_new ("add_view",
 					  GTK_RUN_LAST,
 					  object_class->type,
 					  GTK_SIGNAL_OFFSET (GnomeMDIClass, add_view),
 					  gnome_mdi_marshal_2,
-					  GTK_TYPE_BOOL, 1, GTK_TYPE_POINTER);
+					  GTK_TYPE_BOOL, 1, gtk_widget_get_type());
   mdi_signals[REMOVE_VIEW] = gtk_signal_new ("remove_view",
 					     GTK_RUN_LAST,
 					     object_class->type,
 					     GTK_SIGNAL_OFFSET (GnomeMDIClass, remove_view),
 					     gnome_mdi_marshal_2,
-					     GTK_TYPE_BOOL, 1, GTK_TYPE_POINTER);
+					     GTK_TYPE_BOOL, 1, gtk_widget_get_type());
   mdi_signals[CHILD_CHANGED] = gtk_signal_new ("child_changed",
 					     GTK_RUN_LAST,
 					     object_class->type,
 					     GTK_SIGNAL_OFFSET (GnomeMDIClass, child_changed),
 					     gnome_mdi_marshal_3,
-					     GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+					     GTK_TYPE_NONE, 1, gnome_mdi_child_get_type());
+  mdi_signals[VIEW_CHANGED] = gtk_signal_new ("view_changed",
+					      GTK_RUN_LAST,
+					      object_class->type,
+					      GTK_SIGNAL_OFFSET(GnomeMDIClass, view_changed),
+					      gnome_mdi_marshal_3,
+					      GTK_TYPE_NONE, 1, gtk_widget_get_type());
   mdi_signals[APP_CREATED] = gtk_signal_new ("app_created",
 					     GTK_RUN_LAST,
 					     object_class->type,
 					     GTK_SIGNAL_OFFSET (GnomeMDIClass, app_created),
 					     gnome_mdi_marshal_3,
-					     GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+					     GTK_TYPE_NONE, 1, gnome_app_get_type());
 
   gtk_object_class_add_signals (object_class, mdi_signals, LAST_SIGNAL);
 
@@ -225,6 +232,7 @@ static void gnome_mdi_class_init (GnomeMDIClass *class) {
   class->add_view = NULL;
   class->remove_view = NULL;
   class->child_changed = NULL;
+  class->view_changed = NULL;
   class->app_created = NULL;
 
   parent_class = gtk_type_class (gtk_object_get_type ());
@@ -391,26 +399,14 @@ static GtkWidget *find_item_by_label(GtkMenuShell *shell, gchar *label) {
 
 static void child_list_activated_cb(GtkWidget *w, GnomeMDI *mdi) {
   GnomeMDIChild *child;
-  GtkWindow *window;
 
   child = gtk_object_get_data(GTK_OBJECT(w), MDI_CHILD_KEY);
 
   if( child && (child != mdi->active_child) ) {
     if(child->views)
-      app_set_view(mdi, gnome_mdi_get_app_from_view(child->views->data), GTK_WIDGET(child->views->data));
+      gnome_mdi_set_active_view(mdi, child->views->data);
     else
       gnome_mdi_add_view(mdi, child);
-
-    window = GTK_WINDOW(gnome_mdi_get_app_from_view(mdi->active_view));
-
-    if(mdi->mode == GNOME_MDI_NOTEBOOK)
-      set_page_by_widget(GTK_NOTEBOOK(GNOME_APP(window)->contents), mdi->active_view);
-    
-    /* TODO: hmmm... I dont know how to give focus to the window, so that it would
-       receive keyboard events */
-    gdk_window_raise(GTK_WIDGET(window)->window);
-    gtk_window_set_focus(window, mdi->active_view);
-    gtk_window_activate_focus(window);
   }
 }
 
@@ -437,7 +433,7 @@ static void child_list_menu_create(GnomeMDI *mdi, GnomeApp *app) {
   gtk_widget_queue_resize(submenu);
 }
 
-static void child_list_menu_remove_item(GnomeMDI *mdi, GnomeMDIChild *child) {
+void child_list_menu_remove_item(GnomeMDI *mdi, GnomeMDIChild *child) {
   GtkWidget *item, *shell;
   GnomeApp *app;
   GList *app_node;
@@ -462,7 +458,7 @@ static void child_list_menu_remove_item(GnomeMDI *mdi, GnomeMDIChild *child) {
   }
 }
 
-static void child_list_menu_add_item(GnomeMDI *mdi, GnomeMDIChild *child) {
+void child_list_menu_add_item(GnomeMDI *mdi, GnomeMDIChild *child) {
   GtkWidget *item, *submenu;
   GnomeApp *app;
   GList *app_node;
@@ -573,11 +569,11 @@ static void toplevel_focus(GnomeApp *app, GdkEventFocus *event, GnomeMDI *mdi) {
   mdi->active_window = app;
 
   if((mdi->mode == GNOME_MDI_TOPLEVEL) || (mdi->mode == GNOME_MDI_MODAL))
-    gnome_mdi_set_active_view(mdi, app->contents);
+    set_active_view(mdi, app->contents);
   else if((mdi->mode == GNOME_MDI_NOTEBOOK) && GTK_NOTEBOOK(app->contents)->cur_page)
-    gnome_mdi_set_active_view(mdi, GTK_NOTEBOOK(app->contents)->cur_page->child);
+    set_active_view(mdi, GTK_NOTEBOOK(app->contents)->cur_page->child);
   else
-    gnome_mdi_set_active_view(mdi, NULL);
+    set_active_view(mdi, NULL);
 }
 
 static void book_drag_begin(GtkNotebook *book, GdkEvent *event, GnomeMDI *mdi) {
@@ -648,7 +644,13 @@ static void rootwin_drop(GtkWidget *rw, GdkEvent *event, GnomeMDI *mdi) {
     return;
 
   if(old_book->cur_page) {
+    gint width, height;
+
     view = old_book->cur_page->child;
+
+    width = view->allocation.width;
+    height = view->allocation.height;
+
     gtk_container_remove(GTK_CONTAINER(old_book), view);
 
     app_create(mdi);
@@ -658,6 +660,8 @@ static void rootwin_drop(GtkWidget *rw, GdkEvent *event, GnomeMDI *mdi) {
     book_add_view(GTK_NOTEBOOK(new_book), view);
 
     gtk_window_position(GTK_WINDOW(mdi->active_window), GTK_WIN_POS_MOUSE);
+    
+    gtk_widget_set_usize (view, width, height);
 
     gtk_widget_show(GTK_WIDGET(mdi->active_window));
   }
@@ -822,7 +826,7 @@ static void app_set_view(GnomeMDI *mdi, GnomeApp *app, GtkWidget *view) {
     /* create new child-specific menus */
     if( child->menu_template &&
         ( (ui_info = copy_ui_info_tree(child->menu_template)) != NULL) ) {
-      gnome_app_insert_menus(app, mdi->child_menu_path, ui_info);
+      gnome_app_insert_menus_with_data(app, mdi->child_menu_path, ui_info, child);
       gtk_object_set_data(GTK_OBJECT(app), GNOME_MDI_CHILD_MENU_INFO_KEY, ui_info);
       gtk_object_set_data(GTK_OBJECT(app), ITEM_COUNT_KEY, (gpointer)count_ui_info_items(ui_info));
     }
@@ -854,7 +858,7 @@ static void app_set_view(GnomeMDI *mdi, GnomeApp *app, GtkWidget *view) {
     gtk_object_set_data(GTK_OBJECT(app), ITEM_COUNT_KEY, NULL);
   }
 
-  gnome_mdi_set_active_view(mdi, view);
+  set_active_view(mdi, view);
 }
 
 static void app_destroy(GnomeApp *app) {
@@ -911,12 +915,10 @@ static void app_create(GnomeMDI *mdi) {
   gtk_signal_connect(GTK_OBJECT(window), "destroy",
                      GTK_SIGNAL_FUNC(app_destroy), NULL);
 
-  /* gtk_window_position (GTK_WINDOW(window), GTK_WIN_POS_MOUSE); */
-
   /* set up menus */
   if(mdi->menu_template) {
     ui_info = copy_ui_info_tree(mdi->menu_template);
-    gnome_app_create_menus(GNOME_APP(window), ui_info);
+    gnome_app_create_menus_with_data(GNOME_APP(window), ui_info, mdi);
     gtk_object_set_data(GTK_OBJECT(window), GNOME_MDI_MENUBAR_INFO_KEY, ui_info);
   }
   else {
@@ -934,7 +936,7 @@ static void app_create(GnomeMDI *mdi) {
   /* create toolbar */
   if(mdi->toolbar_template) {
     ui_info = copy_ui_info_tree(mdi->toolbar_template);
-    gnome_app_create_toolbar(GNOME_APP(window), ui_info);
+    gnome_app_create_toolbar_with_data(GNOME_APP(window), ui_info, mdi);
     gtk_object_set_data(GTK_OBJECT(window), GNOME_MDI_TOOLBAR_INFO_KEY, ui_info);
   }
   else {
@@ -970,19 +972,52 @@ static void top_add_view(GnomeMDI *mdi, GnomeMDIChild *child, GtkWidget *view) {
     gtk_widget_show(GTK_WIDGET(window));
 }
 
-static void gnome_mdi_set_active_view(GnomeMDI *mdi, GtkWidget *view) {
-  GnomeMDIChild *child;
+static void set_active_view(GnomeMDI *mdi, GtkWidget *view) {
+  GnomeMDIChild *child, *old_child;
+  GtkWidget *old_view;
+
+#ifdef GNOME_ENABLE_DEBUG
+  g_message("GnomeMDI: set_active_view() called");
+#endif
+
+  if(view == mdi->active_view)
+    return;
 
   if(view)
     child = gnome_mdi_get_child_from_view(view);
   else
     child = NULL;
 
-  if(child != mdi->active_child)
-    gtk_signal_emit(GTK_OBJECT(mdi), mdi_signals[CHILD_CHANGED], child);
+  old_child = mdi->active_child;
+  old_view = mdi->active_view;
 
   mdi->active_view = view;
   mdi->active_child = child;
+
+  if(child != old_child)
+    gtk_signal_emit(GTK_OBJECT(mdi), mdi_signals[CHILD_CHANGED], old_child);
+
+  gtk_signal_emit(GTK_OBJECT(mdi), mdi_signals[VIEW_CHANGED], old_view);
+}
+
+void gnome_mdi_set_active_view(GnomeMDI *mdi, GtkWidget *view) {
+  GtkWindow *window;
+
+  g_return_if_fail(mdi != NULL);
+  g_return_if_fail(GNOME_IS_MDI(mdi));
+  g_return_if_fail(view != NULL);
+  g_return_if_fail(GTK_IS_WIDGET(view));
+
+  window = GTK_WINDOW(gnome_mdi_get_app_from_view(view));
+
+  if(mdi->mode == GNOME_MDI_NOTEBOOK)
+    set_page_by_widget(GTK_NOTEBOOK(GNOME_APP(window)->contents), view);
+  
+  /* TODO: hmmm... I dont know how to give focus to the window, so that it would
+     receive keyboard events */
+  gdk_window_raise(GTK_WIDGET(window)->window);
+  gtk_window_set_focus(window, view);
+  gtk_window_activate_focus(window);
 }
 
 gint gnome_mdi_add_view(GnomeMDI *mdi, GnomeMDIChild *child) {
@@ -1054,9 +1089,6 @@ gint gnome_mdi_remove_view(GnomeMDI *mdi, GtkWidget *view, gint force) {
 
   window = gnome_mdi_get_app_from_view(view);
 
-  if(view == mdi->active_view)
-    app_set_view(mdi, window, NULL);
-
 #if 0
   if(mdi->mode == GNOME_MDI_MS)
     gnome_mdi_pouch_remove(GNOME_MDI_POUCH(GTK_WIDGET(parent)->parent), view);
@@ -1074,14 +1106,19 @@ gint gnome_mdi_remove_view(GnomeMDI *mdi, GtkWidget *view, gint force) {
       mdi->windows = g_list_remove(mdi->windows, window);
       gtk_widget_destroy(GTK_WIDGET(window));
     }
+    else
+      app_set_view(mdi, window, NULL);
   }
   else if( (mdi->mode == GNOME_MDI_NOTEBOOK) &&
-	   (GTK_NOTEBOOK(parent)->cur_page == NULL) &&
-	   (g_list_length(mdi->windows) > 1) ) {
-    /* if this is NOT the last toplevel, destroy it! */
-    mdi->windows = g_list_remove(mdi->windows, window);
-    gtk_widget_destroy(GTK_WIDGET(window));
-  }
+	   (GTK_NOTEBOOK(parent)->cur_page == NULL) ) {
+    if(g_list_length(mdi->windows) > 1) {
+      /* if this is NOT the last toplevel, destroy it! */
+      mdi->windows = g_list_remove(mdi->windows, window);
+      gtk_widget_destroy(GTK_WIDGET(window));
+    }
+    else
+      app_set_view(mdi, window, NULL);
+  }  
 
   /* remove this view from the child's view list */
   gnome_mdi_child_remove_view(child, view);
@@ -1230,6 +1267,7 @@ void gnome_mdi_set_mode(GnomeMDI *mdi, GnomeMDIMode mode) {
   GtkWidget *view;
   GnomeMDIChild *child;
   GList *child_node, *view_node;
+  gint width = -1, height = -1;
 
   g_return_if_fail(mdi != NULL);
   g_return_if_fail(GNOME_IS_MDI(mdi));
@@ -1242,6 +1280,12 @@ void gnome_mdi_set_mode(GnomeMDI *mdi, GnomeMDIMode mode) {
   else if(mdi->mode == mode)
     return;
 
+  /* Get current width and height. */
+  if (mdi->active_view) {
+    width = mdi->active_view->allocation.width;
+    height = mdi->active_view->allocation.height;
+  }
+
   /* remove all views from their parents */
   child_node = mdi->children;
   while(child_node) {
@@ -1294,6 +1338,9 @@ void gnome_mdi_set_mode(GnomeMDI *mdi, GnomeMDIMode mode) {
     while(view_node) {
       view = GTK_WIDGET(view_node->data);
 
+      if ((width != -1) && (height != -1))
+	gtk_widget_set_usize (view, width, height);
+
       if(mdi->mode == GNOME_MDI_NOTEBOOK)
         book_add_view(GTK_NOTEBOOK(mdi->active_window->contents), view);
       else if(mdi->mode == GNOME_MDI_TOPLEVEL)
@@ -1332,6 +1379,13 @@ GnomeMDIChild *gnome_mdi_active_child(GnomeMDI *mdi) {
   return NULL;
 }
 
+GtkWidget *gnome_mdi_active_view(GnomeMDI *mdi) {
+  g_return_val_if_fail(mdi != NULL, NULL);
+  g_return_val_if_fail(GNOME_IS_MDI(mdi), NULL);
+
+  return mdi->active_view;
+}
+
 void gnome_mdi_set_menu_template(GnomeMDI *mdi, GnomeUIInfo *menu_tmpl) {
   g_return_if_fail(mdi != NULL);
   g_return_if_fail(GNOME_IS_MDI(mdi));
