@@ -1,15 +1,16 @@
 /* GNOME GUI Library
- * Copyright (C) 1997 the Free Software Foundation
+ * Copyright (C) 1997, 1998 the Free Software Foundation
  *
  * Authors: Miguel de Icaza
  *          Federico Mena
  */
 
+#include <config.h>
 
 #include <gdk_imlib.h>
 #include "libgnome/gnome-defs.h"
 #include <gtk/gtk.h>                  /* These two includes should be remove once everyting */
-#include "libgnome/libgnome.h"        /* is switched to use the GnomePixmap widget.         */
+#include "libgnome/libgnomeP.h"        /* is switched to use the GnomePixmap widget.         */
 #include "gnome-pixmap.h"
 
 
@@ -113,7 +114,7 @@ gnome_pixmap_destroy (GtkObject *object)
 }
 
 GtkWidget *
-gnome_pixmap_new_from_file (char *filename)
+gnome_pixmap_new_from_file (const char *filename)
 {
 	GnomePixmap *gpixmap;
 
@@ -126,7 +127,7 @@ gnome_pixmap_new_from_file (char *filename)
 }
 
 GtkWidget *
-gnome_pixmap_new_from_file_at_size (char *filename, int width, int height)
+gnome_pixmap_new_from_file_at_size (const char *filename, int width, int height)
 {
 	GnomePixmap *gpixmap;
 
@@ -213,6 +214,34 @@ gnome_pixmap_new_from_rgb_d_at_size (unsigned char *data, unsigned char *alpha,
 	return GTK_WIDGET (gpixmap);
 }
 
+GtkWidget *
+gnome_pixmap_new_from_imlib (GdkImlibImage *im)
+{
+	GnomePixmap *gpixmap;
+
+	g_return_val_if_fail(im != NULL, NULL);
+
+	gpixmap = gtk_type_new (gnome_pixmap_get_type ());
+	gnome_pixmap_load_imlib (gpixmap, im);
+
+	return GTK_WIDGET (gpixmap);
+}
+
+GtkWidget *
+gnome_pixmap_new_from_imlib_at_size (GdkImlibImage *im, int width, int height)
+{
+	GnomePixmap *gpixmap;
+
+	g_return_val_if_fail (im != NULL, NULL);
+	g_return_val_if_fail (width > 0, NULL);
+	g_return_val_if_fail (height > 0, NULL);
+
+	gpixmap = gtk_type_new (gnome_pixmap_get_type ());
+	gnome_pixmap_load_imlib_at_size (gpixmap, im, width, height);
+
+	return GTK_WIDGET (gpixmap);
+}
+
 GtkWidget *
 gnome_pixmap_new_from_gnome_pixmap (GnomePixmap *gpixmap_old)
 {
@@ -409,7 +438,7 @@ gnome_pixmap_expose (GtkWidget *widget, GdkEventExpose *event)
 }
 
 static void
-finish_load (GnomePixmap *gpixmap, GdkImlibImage *im, int scaled, int width, int height)
+finish_load (GnomePixmap *gpixmap, GdkImlibImage *im, int scaled, int width, int height, int destroy)
 {
 	if (!im)
 		return;
@@ -422,7 +451,8 @@ finish_load (GnomePixmap *gpixmap, GdkImlibImage *im, int scaled, int width, int
 	gpixmap->pixmap = gdk_imlib_copy_image (im);
 	gpixmap->mask = gdk_imlib_copy_mask (im);
 
-	gdk_imlib_destroy_image (im);
+	if (destroy)
+		gdk_imlib_destroy_image (im);
 
 	if (GTK_WIDGET_REALIZED (gpixmap)) {
 		if (GTK_WIDGET_MAPPED (gpixmap))
@@ -439,14 +469,14 @@ finish_load (GnomePixmap *gpixmap, GdkImlibImage *im, int scaled, int width, int
 }
 
 static void
-load_file (GnomePixmap *gpixmap, char *filename, int scaled, int width, int height)
+load_file (GnomePixmap *gpixmap, const char *filename, int scaled, int width, int height)
 {
 	GdkImlibImage *im;
 
 	free_pixmap_and_mask (gpixmap);
 
 	im = gdk_imlib_load_image (filename);
-	finish_load (gpixmap, im, scaled, width, height);
+	finish_load (gpixmap, im, scaled, width, height, 1);
 }
 
 static void
@@ -457,7 +487,7 @@ load_xpm_d (GnomePixmap *gpixmap, char **xpm_data, int scaled, int width, int he
 	free_pixmap_and_mask (gpixmap);
 
 	im = gdk_imlib_create_image_from_xpm_data (xpm_data);
-	finish_load (gpixmap, im, scaled, width, height);
+	finish_load (gpixmap, im, scaled, width, height, 1);
 }
 
 static void
@@ -470,7 +500,7 @@ load_rgb_d (GnomePixmap *gpixmap, unsigned char *data, unsigned char *alpha,
 	free_pixmap_and_mask (gpixmap);
 
 	im = gdk_imlib_create_image_from_data (data, alpha, rgb_width, rgb_height);
-	finish_load (gpixmap, im, scaled, width, height);
+	finish_load (gpixmap, im, scaled, width, height, 1);
 }
 
 static void
@@ -486,11 +516,11 @@ load_rgb_d_shaped (GnomePixmap *gpixmap, unsigned char *data,
 
 	im = gdk_imlib_create_image_from_data (data, alpha, rgb_width, rgb_height);
 	gdk_imlib_set_image_shape (im, shape_color);
-	finish_load (gpixmap, im, scaled, width, height);
+	finish_load (gpixmap, im, scaled, width, height, 1);
 }
 
 void
-gnome_pixmap_load_file (GnomePixmap *gpixmap, char *filename)
+gnome_pixmap_load_file (GnomePixmap *gpixmap, const char *filename)
 {
 	g_return_if_fail (gpixmap != NULL);
 	g_return_if_fail (GNOME_IS_PIXMAP (gpixmap));
@@ -500,7 +530,7 @@ gnome_pixmap_load_file (GnomePixmap *gpixmap, char *filename)
 }
 
 void
-gnome_pixmap_load_file_at_size (GnomePixmap *gpixmap, char *filename, int width, int height)
+gnome_pixmap_load_file_at_size (GnomePixmap *gpixmap, const char *filename, int width, int height)
 {
 	g_return_if_fail (gpixmap != NULL);
 	g_return_if_fail (GNOME_IS_PIXMAP (gpixmap));
@@ -578,194 +608,26 @@ gnome_pixmap_load_rgb_d_at_size (GnomePixmap *gpixmap, unsigned char *data, unsi
 	load_rgb_d (gpixmap, data, alpha, rgb_width, rgb_height, TRUE, width, height);
 }
 
-
-/* FIXME:  These are the old gnome-pixmap API functions.  They should
- * be flushed out in favor of the GnomePixmap widget.
- */
-
-struct pixmap_item {
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
-};
-
-static GHashTable *pixmap_hash;
-
-
-static void
-destroy_hash_element (void *key, void *val, void *data)
+void
+gnome_pixmap_load_imlib (GnomePixmap *gpixmap, GdkImlibImage *im)
 {
-	struct pixmap_item *pi = val;
+	g_return_if_fail (gpixmap != NULL);
+	g_return_if_fail (GNOME_IS_PIXMAP (gpixmap));
+	g_return_if_fail (im != NULL);
 
-	gdk_pixmap_unref (pi->pixmap);
-	gdk_pixmap_unref (pi->mask);
-	g_free (val);
-	g_free (key);
+	finish_load (gpixmap, im, 0, 0, 0, 0);
 }
 
 void
-gnome_destroy_pixmap_cache (void)
+gnome_pixmap_load_imlib_at_size (GnomePixmap *gpixmap,
+				 GdkImlibImage *im, int width, int height)
 {
-	if (!pixmap_hash)
-		return;
+	g_return_if_fail (gpixmap != NULL);
+	g_return_if_fail (GNOME_IS_PIXMAP (gpixmap));
+	g_return_if_fail (im != NULL);
+	g_return_if_fail (width > 0);
+	g_return_if_fail (height > 0);
 
-	g_hash_table_foreach (pixmap_hash, destroy_hash_element, 0);
-	g_hash_table_destroy (pixmap_hash);
-	pixmap_hash = NULL;
+	finish_load (gpixmap, im, 1, width, height, 0);
 }
 
-static void
-check_hash_table(void)
-{
-	if (!pixmap_hash)
-		pixmap_hash = g_hash_table_new (g_str_hash, g_str_equal);
-}
-
-static void
-load_pixmap(GdkWindow *window, GdkPixmap **pixmap, GdkBitmap **mask, GdkColor *transparent, char *filename)
-{
-	struct pixmap_item *pit;
-
-	check_hash_table();
-
-	pit = g_hash_table_lookup(pixmap_hash, filename);
-
-	if (!pit) {
-		pit = g_new(struct pixmap_item, 1);
-		pit->pixmap = gdk_pixmap_create_from_xpm(window, &pit->mask, transparent, filename);
-		
-		g_hash_table_insert(pixmap_hash, g_strdup(filename), pit);
-	}
-	
-	*pixmap = pit->pixmap;
-	*mask   = pit->mask;
-}
-
-void
-gnome_create_pixmap_gdk (GdkWindow *window, GdkPixmap **pixmap, GdkBitmap **mask, GdkColor *transparent, char *file)
-{
-	printf ("gnome_create_pixmap_gdk: this function is obsolete and evil; please use gdk_imlib instead\n");
-
-	g_assert(window != NULL);
-	g_assert(pixmap != NULL);
-	g_assert(mask != NULL);
-	g_assert(transparent != NULL);
-
-	if (!file || !g_file_exists(file)) {
-		*pixmap = NULL;
-		*mask = NULL;
-		return;
-	}
-
-	load_pixmap(window, pixmap, mask, transparent, file);
-}
-
-void
-gnome_create_pixmap_gtk (GtkWidget *window, GdkPixmap **pixmap, GdkBitmap **mask, GtkWidget *holder, char *file)
-{
-	GtkStyle *style;
-	
-	printf ("gnome_create_pixmap_gtk: this function is obsolete and evil; please use gdk_imlib instead\n");
-
-	g_assert(window != NULL);
-	g_assert(pixmap != NULL);
-	g_assert(mask != NULL);
-	g_assert(holder != NULL);
-
-	if (!file || !g_file_exists(file)) {
-		*pixmap = NULL;
-		*mask = NULL;
-		return;
-	}
-
-	if (!GTK_WIDGET_REALIZED(window))
-		gtk_widget_realize(window);
-
-	style = gtk_widget_get_style (holder);
-
-	load_pixmap(window->window, pixmap, mask,
-		    style ? &style->bg[GTK_STATE_NORMAL] : NULL, /* XXX: NULL will make it bomb */
-		    file);
-}
-
-void
-gnome_create_pixmap_gtk_d (GtkWidget *window, GdkPixmap **pixmap, GdkBitmap **mask, GtkWidget *holder,
-			   char **data)
-{
-	GtkStyle *style;
-	
-	printf ("gnome_create_pixmap_gtk_d: this function is obsolete and evil; please use gdk_imlib instead\n");
-
-	g_assert(window != NULL);
-	g_assert(pixmap != NULL);
-	g_assert(mask != NULL);
-	g_assert(holder != NULL);
-
-	if (!GTK_WIDGET_REALIZED(window))
-		gtk_widget_realize(window);
-
-	style = gtk_widget_get_style (holder);
-
-	*pixmap = gdk_pixmap_create_from_xpm_d (window->window, mask, &style->bg[GTK_STATE_NORMAL], data);
-}
-
-GtkWidget *
-gnome_create_pixmap_widget (GtkWidget *window, GtkWidget *holder, char *file)
-{
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
-
-	printf ("gnome_create_pixmap_widget: this function is obsolete and evil; please use GnomePixmap instead\n");
-
-	if (!file || !g_file_exists(file))
-		return NULL;
-
-	gnome_create_pixmap_gtk(window, &pixmap, &mask, holder, file);
-
-	return gtk_pixmap_new(pixmap, mask);
-}
-
-GtkWidget *
-gnome_create_pixmap_widget_d (GtkWidget *window, GtkWidget *holder, char **data)
-{
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
-
-	printf ("gnome_create_pixmap_widget_d: this function is obsolete and evil; please use GnomePixmap instead\n");
-
-	gnome_create_pixmap_gtk_d(window, &pixmap, &mask, holder, data);
-
-	return gtk_pixmap_new(pixmap, mask);
-}
-
-void
-gnome_set_pixmap_widget (GtkPixmap *pixmap, GtkWidget *window, GtkWidget *holder, gchar *file)
-{
-	GdkPixmap *gpixmap;
-	GdkBitmap *mask;
-
-	printf ("gnome_set_pixmap_widget: this function is obsolete and evil; please use gdk_imlib instead\n");
-	
-	g_assert (pixmap != NULL);
-
-	if (!file || !g_file_exists(file))
-		return;
-
-	gnome_create_pixmap_gtk(window, &gpixmap, &mask, holder, file);
-
-	gtk_pixmap_set(pixmap, gpixmap, mask);
-}
-
-void
-gnome_set_pixmap_widget_d (GtkPixmap *pixmap, GtkWidget *window, GtkWidget *holder, gchar **data)
-{
-	GdkPixmap *gpixmap;
-	GdkBitmap *mask;
-	
-	printf ("gnome_set_pixmap_widget_d: this function is obsolete and evil; please use gdk_imlib instead\n");
-	
-	g_assert (pixmap != NULL);
-
-	gnome_create_pixmap_gtk_d(window, &gpixmap, &mask, holder, data);
-
-	gtk_pixmap_set(pixmap, gpixmap, mask);
-}
