@@ -36,6 +36,7 @@ struct _GnomePreferences {
   int menubar_handlebox : 1;
   int toolbar_relief : 1;
   int dialog_centered : 1;
+  int menus_have_icons : 1;
   GtkWindowType dialog_type;
   GtkWindowPosition dialog_position;
   GnomeMDIMode mdi_mode;
@@ -60,6 +61,7 @@ static GnomePreferences prefs =
   TRUE,               /* Menubar has handlebox */
   TRUE,               /* Toolbar buttons are relieved */
   TRUE,               /* Center dialogs over apps when possible */
+  TRUE,               /* Menu items have icons in them */
   GTK_WINDOW_DIALOG,  /* Dialogs are treated specially */
   GTK_WIN_POS_MOUSE,  /* Put dialogs at mouse pointer. */
   GNOME_MDI_NOTEBOOK, /* Use notebook MDI mode. */
@@ -134,6 +136,8 @@ static const gchar * const dialog_positions [] = {
 
 #define TOOLBAR_RELIEF_KEY         "Toolbar_relieved_buttons"
 
+#define MENUS_HAVE_ICONS_KEY       "Menus_have_icons"
+
 /* =========== MDI ================================= */
 
 #define MDI_MODE_KEY               "MDI_mode"
@@ -177,14 +181,14 @@ enum_from_strings(gint * setme, gchar * findme,
     }
   }
 
-  g_free(findme);
   return retval;
 }
 
 void gnome_preferences_load(void)
 {
   /* Probably this function should be rewritten to use the 
-     _preferences_get functions */
+   *  _preferences_get functions
+   */
   gboolean b;
   gchar * s;
 
@@ -197,6 +201,8 @@ void gnome_preferences_load(void)
 			   dialog_button_styles, NUM_BUTTON_STYLES) ) {
     g_warning("Didn't recognize buttonbox style in libgnomeui config");
   }
+
+  g_free(s);
   
   s = gnome_config_get_string(DIALOG_TYPE_KEY);
 
@@ -205,6 +211,8 @@ void gnome_preferences_load(void)
     g_warning("Didn't recognize dialog type in libgnomeui config");
   }
 
+  g_free(s);
+  
   s = gnome_config_get_string(DIALOG_POSITION_KEY);
 
   if ( ! enum_from_strings((int*) &prefs.dialog_position, s,
@@ -212,6 +220,8 @@ void gnome_preferences_load(void)
     g_warning("Didn't recognize dialog position in libgnomeui config");
   }  
 
+  g_free(s);
+  
   /* Fixme. There's a little problem with no error value from the 
      bool get function. This makes it yucky to do the propertybox 
      thing. The intermediate 'b' variable is only in anticipation
@@ -221,7 +231,6 @@ void gnome_preferences_load(void)
 					 NULL);
   prefs.dialog_centered = b;
 
-#if 0
   /* This is unused for now */
   b = gnome_config_get_bool_with_default(PROPERTY_BOX_BUTTONS_OK_KEY"=true",
 					 NULL);
@@ -238,7 +247,6 @@ void gnome_preferences_load(void)
   b = gnome_config_get_bool_with_default(PROPERTY_BOX_BUTTONS_HELP_KEY"=true",
 					 NULL);
   prefs.property_box_buttons_help = b;
-#endif
 
   gnome_config_pop_prefix();
   gnome_config_push_prefix(STATUSBAR);
@@ -268,6 +276,10 @@ void gnome_preferences_load(void)
 					 NULL);
   prefs.toolbar_relief = b;
 
+  b = gnome_config_get_bool_with_default (MENUS_HAVE_ICONS_KEY"=true",
+					  NULL);
+  prefs.menus_have_icons = b;
+
   gnome_config_pop_prefix();
   gnome_config_push_prefix(MDI);
 
@@ -278,6 +290,8 @@ void gnome_preferences_load(void)
     g_warning("Didn't recognize MDI mode in libgnomeui config");
   }
 
+  g_free(s);
+  
   s = gnome_config_get_string(MDI_TAB_POS_KEY);
 
   if ( ! enum_from_strings((int*) &prefs.mdi_tab_pos, s,
@@ -285,6 +299,8 @@ void gnome_preferences_load(void)
     g_warning("Didn't recognize MDI notebook tab position in libgnomeui config");
   }
 
+  g_free(s);
+  
   gnome_config_pop_prefix();
 }
 
@@ -323,6 +339,8 @@ void gnome_preferences_save(void)
 			prefs.menubar_handlebox);
   gnome_config_set_bool(TOOLBAR_RELIEF_KEY,
 			prefs.toolbar_relief);
+  gnome_config_set_bool(MENUS_HAVE_ICONS_KEY,
+			prefs.menus_have_icons);
 
   gnome_config_pop_prefix();
   gnome_config_push_prefix(MDI);
@@ -448,3 +466,26 @@ void              gnome_preferences_set_mdi_tab_pos          (GtkPositionType p)
   prefs.mdi_tab_pos = p;
 }
 
+int
+gnome_preferences_get_property_box_apply (void)
+{
+	return prefs.property_box_buttons_apply;
+}
+
+void
+gnome_preferences_set_property_box_button_apply (int v)
+{
+	prefs.property_box_buttons_apply = v;
+}
+
+int
+gnome_preferences_get_menus_have_icons (void)
+{
+	return prefs.menus_have_icons;
+}
+
+void
+gnome_preferences_set_menus_have_icons (int have_icons)
+{
+	prefs.menus_have_icons = have_icons ? TRUE : FALSE;
+}
