@@ -57,7 +57,7 @@
 
 
 
-#include "gnome-stock-imlib.h"
+#include "pixmaps/gnome-stock-imlib.h"
 
 
 
@@ -311,6 +311,9 @@ gnome_stock_pixmap_widget_set_icon(GnomeStockPixmapWidget *widget,
 /*  new GnomeStock widget  */
 /***************************/
 
+static gint (*parent_expose)(GtkWidget *widget, GdkEventExpose *event) = NULL;
+static void (*parent_draw)(GtkWidget *widget, GdkRectangle *area) = NULL;
+
 static void
 gnome_stock_destroy(GtkObject *object)
 {
@@ -359,9 +362,9 @@ gnome_stock_paint(GnomeStock *stock, GnomePixmap *pixmap)
 		gint width, height;
 		gdk_window_get_size(gpixmap->pixmap, &width, &height);
 		if ((width != req.width) || (height != req.height)) {
-			gdk_window_destroy(gpixmap->pixmap);
+			gdk_pixmap_unref(gpixmap->pixmap);
 			gpixmap->pixmap = NULL;
-			gdk_window_destroy(gpixmap->mask);
+			gdk_pixmap_unref(gpixmap->mask);
 			gpixmap->mask = NULL;
 		}
 	}
@@ -376,13 +379,18 @@ gnome_stock_paint(GnomeStock *stock, GnomePixmap *pixmap)
 	gdk_draw_pixmap(gpixmap->pixmap, gc, pixmap->pixmap, 0, 0, 0, 0,
 			req.width, req.height);
 	gdk_gc_destroy(gc);
-	gc = gdk_gc_new(gpixmap->mask);
-	gdk_draw_pixmap(gpixmap->mask, gc, pixmap->mask, 0, 0, 0, 0,
-			req.width, req.height);
-	gdk_gc_destroy(gc);
+	if (pixmap->mask) {
+		gc = gdk_gc_new(gpixmap->mask);
+		gdk_draw_pixmap(gpixmap->mask, gc, pixmap->mask, 0, 0, 0, 0,
+				req.width, req.height);
+		gdk_gc_destroy(gc);
+	}
 	if (GTK_WIDGET(gpixmap)->window)
 		gdk_window_shape_combine_mask(GTK_WIDGET(gpixmap)->window,
 					      gpixmap->mask, 0, 0);
+	/* XXX: this isn't needed always, but I found that in some
+	 * circumstances it is. */
+	gtk_widget_queue_draw(GTK_WIDGET(stock));
 }
 
 /*
@@ -396,34 +404,59 @@ gnome_stock_state_changed(GtkWidget *widget, guint prev_state)
 	GnomeStock *w = GNOME_STOCK(widget);
 	GnomePixmap *pixmap;
 	GtkWidget *tmp;
+	static GHashTable *hash = NULL;
+	struct style_hash {
+		GtkWidget *widget;
+	};
 
+	if (!hash) {
+		hash = g_hash_table_new(g_direct_hash, g_direct_equal);
+	}
 	if (widget->parent) {
+		GdkWindow *w;
 		tmp = widget->parent;
-		if (tmp->parent)
+		w = tmp->window;
+		while (tmp->parent) {
+			if (tmp->parent->window != w) break;
 			tmp = tmp->parent;
+		}
 	} else {
 		tmp = widget;
 	}
+	if (g_hash_table_lookup(hash, widget)) {
+		if (tmp != g_hash_table_lookup(hash, widget)) {
+			if (w->regular) {
+				gtk_widget_unref(GTK_WIDGET(w->regular));
+				w->regular = NULL;
+			}
+			if (w->disabled) {
+				gtk_widget_unref(GTK_WIDGET(w->disabled));
+				w->disabled = NULL;
+			}
+			if (w->focused) {
+				gtk_widget_unref(GTK_WIDGET(w->focused));
+				w->focused = NULL;
+			}
+			w->current = NULL;
+		}
+		/* FIXME: do I really need to remove the entry? */
+		g_hash_table_remove(hash, widget);
+	}
+	g_hash_table_insert(hash, widget, tmp);
 	pixmap = NULL;
 	if (GTK_WIDGET_HAS_FOCUS(widget)) {
 		if (!w->focused) {
 			w->focused = gnome_stock_pixmap(tmp, w->icon, GNOME_STOCK_PIXMAP_FOCUSED);
-			/* gtk_widget_ref(GTK_WIDGET(w->focused)); */
-			/* gtk_widget_show(GTK_WIDGET(w->focused)); */
 		}
 		pixmap = w->focused;
 	} else if (!GTK_WIDGET_IS_SENSITIVE(widget)) {
 		if (!w->disabled) {
 			w->disabled = gnome_stock_pixmap(tmp, w->icon, GNOME_STOCK_PIXMAP_DISABLED);
-			/* gtk_widget_ref(GTK_WIDGET(w->disabled)); */
-			/* gtk_widget_show(GTK_WIDGET(w->disabled)); */
 		}
 		pixmap = w->disabled;
 	} else {
 		if (!w->regular) {
 			w->regular = gnome_stock_pixmap(tmp, w->icon, GNOME_STOCK_PIXMAP_REGULAR);
-			/* gtk_widget_ref(GTK_WIDGET(w->regular)); */
-			/* gtk_widget_show(GTK_WIDGET(w->regular)); */
 		}
 		pixmap = w->regular;
 	}
@@ -433,12 +466,32 @@ gnome_stock_state_changed(GtkWidget *widget, guint prev_state)
 	gnome_stock_paint(w, pixmap);
 }
 
+static gint
+gnome_stock_expose(GtkWidget *widget, GdkEventExpose *event)
+{
+	gnome_stock_state_changed(widget, 0);
+	return (*parent_expose)(widget, event);
+}
+
+static void
+gnome_stock_draw(GtkWidget *widget, GdkRectangle *area)
+{
+	gnome_stock_state_changed(widget, 0);
+	(*parent_draw)(widget, area);
+}
+
 static void
 gnome_stock_class_init(GtkObjectClass *klass)
 {
 	klass->destroy = gnome_stock_destroy;
 	((GtkWidgetClass *)klass)->state_changed =
 		gnome_stock_state_changed;
+	parent_expose = ((GtkWidgetClass *)klass)->expose_event;
+	((GtkWidgetClass *)klass)->expose_event =
+		gnome_stock_expose;
+	parent_draw = ((GtkWidgetClass *)klass)->draw;
+	((GtkWidgetClass *)klass)->draw =
+		gnome_stock_draw;
 }
 
 static void
@@ -463,7 +516,8 @@ gnome_stock_get_type(void)
 			(GtkClassInitFunc)gnome_stock_class_init,
 			(GtkObjectInitFunc)gnome_stock_init,
 			(GtkArgSetFunc) NULL,
-			(GtkArgGetFunc) NULL
+			(GtkArgGetFunc) NULL,
+			(GtkClassInitFunc) NULL
 		};
 		new_type = gtk_type_unique(gnome_pixmap_get_type(), &type_info);
 	}
@@ -649,7 +703,7 @@ scale_down(GtkWidget *window, unsigned char *datao, gint wo, gint ho, gint w, gi
 struct _default_entries_data {
 	char *icon, *subtype;
 	char *label;
-	gchar *rgb_data;
+	const gchar *rgb_data;
 	int width, height;
 	int scaled_width, scaled_height;
 };
@@ -708,7 +762,9 @@ struct _default_entries_data entries_data[] = {
 	{GNOME_STOCK_PIXMAP_BOOK_YELLOW, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_book_yellow, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
 	{GNOME_STOCK_PIXMAP_BOOK_OPEN, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_book_open, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
 	{GNOME_STOCK_PIXMAP_NOT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_not, TB_W, TB_H, TB_W, TB_H},
-	{GNOME_STOCK_PIXMAP_MULTIPLE, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_multiple_file, 32, 32 },
+	{GNOME_STOCK_PIXMAP_CONVERT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_convert, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_JUMP_TO, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_jump_to, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_MULTIPLE, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_multiple_file, 32, 32, 32, 32},
 	{GNOME_STOCK_PIXMAP_EXIT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_exit, TB_W, TB_H, TB_W, TB_H},
 	{GNOME_STOCK_PIXMAP_ABOUT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_menu_about, MENU_W, MENU_H, MENU_W, MENU_H},
 	{GNOME_STOCK_BUTTON_OK, GNOME_STOCK_PIXMAP_REGULAR, N_("OK"), imlib_button_ok, TB_W, TB_H, TB_W, TB_H},
@@ -745,6 +801,8 @@ struct _default_entries_data entries_data[] = {
 	{GNOME_STOCK_MENU_STOP, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_stop, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
 	{GNOME_STOCK_MENU_REFRESH, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_refresh, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
 	{GNOME_STOCK_MENU_UNDELETE, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_undelete, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_TIMER, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_timer, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_TIMER_STOP, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_timer_stopped, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
 	{GNOME_STOCK_MENU_MAIL, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_mail, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
 	{GNOME_STOCK_MENU_MAIL_RCV, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_mail_receive, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
 	{GNOME_STOCK_MENU_MAIL_SND, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_mail_send, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
@@ -762,6 +820,8 @@ struct _default_entries_data entries_data[] = {
 	{GNOME_STOCK_MENU_BOOK_BLUE, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_book_blue, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
 	{GNOME_STOCK_MENU_BOOK_YELLOW, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_book_yellow, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
 	{GNOME_STOCK_MENU_BOOK_OPEN, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_book_open, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_CONVERT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_convert, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_JUMP_TO, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_jump_to, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
 	{GNOME_STOCK_MENU_ABOUT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_menu_about, MENU_W, MENU_H, MENU_W, MENU_H},
 	/* TODO: I shouldn't waste a pixmap for that */
 	{GNOME_STOCK_MENU_BLANK, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_menu_blank, MENU_W, MENU_H, MENU_W, MENU_H}, 
@@ -870,7 +930,7 @@ create_pixmap_from_imlib(GtkWidget *window, GnomeStockPixmapEntryImlib *data)
 {
 	static GdkImlibColor shape_color = { 0xff, 0, 0xff, 0 };
 
-	return (GnomePixmap *)gnome_pixmap_new_from_rgb_d_shaped(data->rgb_data,
+	return (GnomePixmap *)gnome_pixmap_new_from_rgb_d_shaped((gchar *)data->rgb_data,
 								 NULL,
 								 data->width,
 								 data->height,
@@ -887,10 +947,11 @@ create_pixmap_from_imlib_scaled(GtkWidget *window,
 
 	if ((data->width != data->scaled_width) ||
 	    (data->height != data->scaled_height)) {
-		d = scale_down(window, data->rgb_data, data->width, data->height,
-			       data->scaled_width, data->scaled_height);
+		d = scale_down(window, (gchar *)data->rgb_data, data->width,
+			       data->height, data->scaled_width,
+			       data->scaled_height);
 	} else {
-		d = data->rgb_data;
+		d = (gchar *)data->rgb_data;
 	}
 
 	return (GnomePixmap *)gnome_pixmap_new_from_rgb_d_shaped(d, NULL,
@@ -1162,7 +1223,8 @@ gnome_pixmap_button(GtkWidget *pixmap, const char *text)
 		    (pixmap->window == NULL))
 			GNOME_STOCK_PIXMAP_WIDGET(pixmap)->window = button;
 #endif /* !USE_NEW_GNOME_STOCK */
-		if (GNOME_IS_PIXMAP(pixmap)) {
+		if ((GNOME_IS_PIXMAP(pixmap)) &&
+		    (!GNOME_IS_STOCK(pixmap))) {
 			GnomeStockPixmapEntry *entry;
 			char s[32];
 
@@ -1261,12 +1323,9 @@ gnome_stock_menu_item(const char *type, const char *text)
 		gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
 
 		menu_item = gtk_menu_item_new();
-#ifdef	GTK_HAVE_FEATURES_1_1_0
+
 		w = gtk_accel_label_new (text);
 		gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (w), menu_item);
-#else	/* !GTK_HAVE_FEATURES_1_1_0 */
-		w = gtk_label_new(text);
-#endif	/* !GTK_HAVE_FEATURES_1_1_0 */
 		gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
 		gtk_widget_show(w);
 		gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
@@ -1318,7 +1377,7 @@ struct default_AccelEntry default_accel_hash[] = {
 	{GNOME_STOCK_MENU_MAIL, {0, 0}},
 	{GNOME_STOCK_MENU_MAIL_RCV, {0, 0}},
 	{GNOME_STOCK_MENU_MAIL_SND, {0, 0}},
-	{NULL}
+	{NULL, {0,0}}
 };
 
 
@@ -1632,6 +1691,8 @@ accel_dlg_select(GtkCList *widget, int row, int col, GdkEventButton *event)
 }
 
 
+/* make the compiler happy */
+void gnome_stock_menu_accel_dlg(char *section);
 
 void
 gnome_stock_menu_accel_dlg(char *section)
@@ -1710,11 +1771,12 @@ gnome_stock_transparent_window (const char *icon, const char *subtype)
 		break;
 	case GNOME_STOCK_PIXMAP_TYPE_IMLIB_SCALED:
 	case GNOME_STOCK_PIXMAP_TYPE_IMLIB:
-		im = gdk_imlib_create_image_from_data (entry->imlib.rgb_data, NULL,
+		im = gdk_imlib_create_image_from_data ((gchar *)entry->imlib.rgb_data, NULL,
 						       entry->imlib.width, entry->imlib.height);
 		break;
 	 default:
-		 break;
+		im = NULL;
+		break;
 	}
 
 	if (!im)
