@@ -19,6 +19,7 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <config.h>
 #include <sys/time.h>
 #include <string.h>
 #include <stdlib.h>
@@ -26,6 +27,9 @@
 #include <stdio.h>
 #include "gtkcalendar.h"
 #include "libgnome/lib_date.h"
+#include <libgnome/gnome-defs.h>
+#include "libgnome/gnome-i18nP.h"
+
 
 #define HELVETICA_10_FONT  "-adobe-helvetica-bold-r-normal-*-10-*-*-*-*-*-*-*"
 #define FIXED_10_FONT      "-b&h-lucidatypewriter-medium-r-normal-*-10-*-*-*-*-*-*-*"
@@ -681,7 +685,7 @@ gtk_calendar_size_request (GtkWidget      *widget,
   /* Calculate max month string width */
   max_month_width = 0;
   for (i = 1; i <= 12; i++) {
-    sprintf (buffer, "%s", month_name[i]);
+    sprintf (buffer, "%s", _(month_name[i]));
     str_width = gdk_string_measure (cal->month_font, buffer);
     if (str_width > max_month_width)
       max_month_width = str_width;
@@ -856,7 +860,7 @@ gtk_calendar_paint_header (GtkWidget *widget)
                    y, buffer);
 
   /* Draw month */ 
-  sprintf (buffer, "%s", month_name[calendar->month + 1]);
+  sprintf (buffer, "%s", _(month_name[calendar->month + 1]));
   str_width = gdk_string_measure (calendar->month_font, buffer);
   gdk_draw_string (calendar->header_win, calendar->month_font, gc, 
                    3 + calendar->arrow_width + (max_month_width - str_width)/2, 
@@ -886,7 +890,7 @@ gtk_calendar_paint_day_names (GtkWidget *widget)
   int day_wid_sep;
   int str_width;
   static char *dayinletters[] =
-  {"Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"};
+  {N_("Su"), N_("Mo"), N_("Tu"), N_("We"), N_("Th"), N_("Fr"), N_("Sa")};
 
   g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_CALENDAR (widget));
@@ -915,7 +919,7 @@ gtk_calendar_paint_day_names (GtkWidget *widget)
   gdk_gc_set_foreground (gc, &calendar->background_color);
   for (day = 0; day < 7; day++)
     {
-      sprintf (buffer, "%s", dayinletters[day]);
+      sprintf (buffer, "%s", _(dayinletters[day]));
       str_width = gdk_string_measure (calendar->day_name_font, buffer);
       gdk_draw_string (calendar->day_name_win, calendar->day_name_font, 
                        gc, CALENDAR_MARGIN + day_wid_sep * day + 
