@@ -19,8 +19,8 @@
 
 #include "gtkcauldron.h"
 #define HAVE_GNOME
-#ifdef HAVE_GNOME
 #include <gtk/gtk.h>
+#ifdef HAVE_GNOME
 #include "libgnomeui/gnome-file-entry.h"
 #include "libgnomeui/gnome-number-entry.h"
 #include "libgnomeui/gnome-stock.h"
@@ -30,6 +30,7 @@
 #include <stdarg.h>
 #include <stdlib.h>
 
+
 gchar *GTK_CAULDRON_ENTER = "GTK_CAULDRON_ENTER";
 gchar *GTK_CAULDRON_ESCAPE = "GTK_CAULDRON_ESCAPE";
 
@@ -40,7 +41,7 @@ enum {
     HORIZONTAL
 };
 
-#define CAULDRON_FMT_OPTIONS "xfpseocrdqjhvag"
+#define CAULDRON_FMT_OPTIONS "xfpseocrdqjhvagn"
 
 struct cauldron_result {
     void (*get_result) (GtkWidget *, void *);
@@ -61,6 +62,8 @@ struct cauldron_button {
 /* loop through all widgets that must return a value - is called before exit */
 static void gtk_cauldron_get_results (struct cauldron_result *r)
 {
+    if (!r)
+	return;
     while (r->prev)
 	r = r->prev;
     while (r) {
@@ -76,7 +79,7 @@ void gtk_cauldron_button_exit (GtkWidget * w, struct cauldron_button *b)
 	gtk_cauldron_get_results (b->r);
     if (b->window) {
 	*(b->result) = b->label;	/* returned from gtk_dialog_cauldron */
-	gtk_widget_destroy (b->window);
+	gtk_main_quit ();
     }
 }
 
@@ -204,31 +207,32 @@ gint space_after (gchar * p)
 
 /* {{{ a simple stack */
 
-static gint widget_stack_pointer = 0;
-static GtkWidget *widget_stack[256] =
-{0};
+typedef struct _GtkCauldronStack {
+    gint widget_stack_pointer;
+    GtkWidget *widget_stack[256];
+} GtkCauldronStack;
 
-static void widget_stack_push (GtkWidget * x)
+static void widget_stack_push (GtkCauldronStack * s, GtkWidget * x)
 {
-    if (widget_stack_pointer >= 250)
+    if (s->widget_stack_pointer >= 250)
 	g_error ("to many widget's in gtk_dialog_cauldron()");
-    widget_stack[widget_stack_pointer++] = x;
-    widget_stack[widget_stack_pointer] = 0;
+    s->widget_stack[s->widget_stack_pointer++] = x;
+    s->widget_stack[s->widget_stack_pointer] = 0;
 }
 
-static GtkWidget *widget_stack_pop (void)
+static GtkWidget *widget_stack_pop (GtkCauldronStack * s)
 {
-    if (widget_stack_pointer <= 0)
+    if (s->widget_stack_pointer <= 0)
 	g_error ("bracketing error in call to gtk_dialog_cauldron()");
-    return widget_stack[--widget_stack_pointer];
+    return s->widget_stack[--(s->widget_stack_pointer)];
 }
 
-static GtkWidget *widget_stack_top (void)
+static GtkWidget *widget_stack_top (GtkCauldronStack * s)
 {
-    if (widget_stack_pointer <= 0)
+    if (s->widget_stack_pointer <= 0)
 	g_error ("bracketing error in call to gtk_dialog_cauldron()");
-    widget_stack[widget_stack_pointer] = 0;
-    return widget_stack[widget_stack_pointer - 1];
+    s->widget_stack[s->widget_stack_pointer] = 0;
+    return s->widget_stack[s->widget_stack_pointer - 1];
 }
 
 /* }}} a simple stack */
@@ -300,9 +304,12 @@ static void gtk_cauldron_box_add (GtkWidget * b, GtkWidget * w, gchar ** p, gint
 	    break;
 	}
     }
-    if (GTK_IS_BOX (b))
+    if (GTK_IS_BOX (b)) {
+/* special hack for notebooks */
+	if (GTK_IS_NOTEBOOK (w))
+	    expand = fill = TRUE;
 	gtk_box_pack_start (GTK_BOX (b), w, expand, fill, padding);
-    else if (GTK_IS_CONTAINER (b))
+    } else if (GTK_IS_CONTAINER (b))
 	gtk_container_add (GTK_CONTAINER (b), w);
 }
 
@@ -325,14 +332,14 @@ static gint key_press_event (GtkWidget * window, GdkEventKey * event, struct key
 	if (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter || event->keyval == GDK_ISO_Enter || event->keyval == GDK_3270_Enter) {
 	    *d->result = GTK_CAULDRON_ENTER;
 	    gtk_cauldron_get_results (*d->r);
-	    gtk_widget_destroy (window);
+	    gtk_main_quit ();
 	    return TRUE;
 	}
     }
     if (!(d->options & GTK_CAULDRON_IGNOREESCAPE)) {
 	if (event->keyval == GDK_Escape) {
 	    *d->result = GTK_CAULDRON_ESCAPE;
-	    gtk_widget_destroy (window);
+	    gtk_main_quit ();
 	    return TRUE;
 	}
     }
@@ -388,7 +395,7 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 {
     GtkWidget *w = 0, *window;
     GSList *radio_group = 0;
-    gchar *p, *q, *result = 0, *fmt;
+    gchar *p, *return_result = 0, *fmt;
     gint pixels_per_space = 3;
     struct cauldron_button b[256];
     gint ncauldron_button = 0;
@@ -396,6 +403,7 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
     struct cauldron_result *r = 0;
     struct key_press_event_data d;
     GtkAccelGroup *accel_table;
+    GtkCauldronStack *stack;
 
 /* text widgets may only add text after the widget is realized,
    so we store text to be added in these, and then add it 
@@ -404,6 +412,9 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
     gint ntext_widget = 0;
     gchar *text[MAX_TEXT_WIDGETS];
 
+    stack = malloc (sizeof (*stack));
+    memset (stack, 0, sizeof (*stack));
+
     if (options & GTK_CAULDRON_SPACE_MASK)
 	pixels_per_space = (options & GTK_CAULDRON_SPACE_MASK) >> GTK_CAULDRON_SPACE_SHIFT;
 
@@ -415,39 +426,32 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
     else if (options & GTK_CAULDRON_POPUP)
 	how = GTK_WINDOW_POPUP;
     window = gtk_window_new (how);
+    gtk_window_position (window, GTK_WIN_POS_MOUSE);
     accel_table = gtk_accel_group_new ();
     gtk_window_add_accel_group (GTK_WINDOW (window), accel_table);
 
     if (title)
 	gtk_window_set_title (GTK_WINDOW (window), title);
 
-    d.result = &result;
+    d.result = &return_result;
     d.options = options;
     d.r = &r;
 
-    gtk_signal_connect (GTK_OBJECT (window), "destroy",
+    gtk_signal_connect (GTK_OBJECT (window), "delete_event",
 			GTK_SIGNAL_FUNC (gtk_main_quit),
 			NULL);
 
     gtk_signal_connect (GTK_OBJECT (window), "key_press_event",
 			(GtkSignalFunc) key_press_event, (gpointer) & d);
 
-    widget_stack_push (window);
+    widget_stack_push (stack, window);
+
+/* add outermost brackets to make parsing easier : */
+    p = fmt = malloc (strlen (format) + 8);
+    strcpy (fmt, "(");
+    strcat (fmt, (gchar *) format);
+    strcat (fmt, ")xf ");
 
-/* add outermost brackets if missing : */
-    p = fmt = (gchar *) format;
-    while (*p == ' ')
-	p++;
-    q = p + strlen (p) - 1;
-    while (*q == ' ')
-	q--;
-    if (*p != '(' || *q != ')') {
-	fmt = alloca (strlen (p) + 3);
-	strcpy (fmt, "(");
-	strcat (fmt, p);
-	strcat (fmt, ")");
-	p = fmt;
-    }
 /* main loop */
     for (; *p; p++) {
 	switch (*p) {
@@ -460,17 +464,17 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 	    switch (find_breaker (p)) {
 	    case VERTICAL:
 	    case VERTICAL_HOMOGENOUS:
-		widget_stack_push (gtk_vpaned_new ());
+		widget_stack_push (stack, gtk_vpaned_new ());
 		break;
 	    case HORIZONTAL:
 	    case HORIZONTAL_HOMOGENOUS:
-		widget_stack_push (gtk_hpaned_new ());
+		widget_stack_push (stack, gtk_hpaned_new ());
 		break;
 	    }
-	    gtk_container_border_width (GTK_CONTAINER (widget_stack_top ()), pixels_per_space * space_after (p));
+	    gtk_container_border_width (GTK_CONTAINER (widget_stack_top (stack)), pixels_per_space * space_after (p));
 	    break;
 	case '(':
-	    {
+	    if (strchr ("% \t([{<>}])", p[1])) {
 		gchar *q;
 		gint s;
 		q = strchr (p, '%');
@@ -479,30 +483,46 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 		s = space_after (q) * pixels_per_space;
 		switch (find_breaker (p)) {
 		case VERTICAL:
-		    widget_stack_push (gtk_vbox_new (FALSE, s));
+		    widget_stack_push (stack, gtk_vbox_new (FALSE, s));
 		    break;
 		case VERTICAL_HOMOGENOUS:
-		    widget_stack_push (gtk_vbox_new (TRUE, s));
+		    widget_stack_push (stack, gtk_vbox_new (TRUE, s));
 		    break;
 		case HORIZONTAL:
-		    widget_stack_push (gtk_hbox_new (FALSE, s));
+		    widget_stack_push (stack, gtk_hbox_new (FALSE, s));
 		    break;
 		case HORIZONTAL_HOMOGENOUS:
-		    widget_stack_push (gtk_hbox_new (TRUE, s));
+		    widget_stack_push (stack, gtk_hbox_new (TRUE, s));
 		    break;
 		}
+	    } else {
+/* place a label right in here */
+		gchar *label;
+		label = strdup (p + 1);
+		p = strchr (p, ')');
+		if (!p) {
+		    g_error ("bracketing error in call to gtk_dialog_cauldron()");
+		    return 0;
+		}
+		*(strchr (label, ')')) = '\0';
+		w = gtk_label_new (label);
+		free (label);
+		user_callbacks (w, p + 1, next_arg, user_data, accel_table);
+		gtk_cauldron_box_add (widget_stack_top (stack), w, &p, pixels_per_space);
+		gtk_widget_show (w);
+		break;
 	    }
-	    gtk_container_border_width (GTK_CONTAINER (widget_stack_top ()), pixels_per_space * space_after (p));
+	    gtk_container_border_width (GTK_CONTAINER (widget_stack_top (stack)), pixels_per_space * space_after (p));
 	    break;
 	case '[':
-	    widget_stack_push (gtk_frame_new (NULL));
+	    widget_stack_push (stack, gtk_frame_new (NULL));
 	    break;
 	case '%':
 	    switch (c = *(++p)) {
 	    case '[':{
 		    char *label;
 		    next_arg (GTK_CAULDRON_TYPE_CHAR_P, user_data, &label);
-		    widget_stack_push (gtk_frame_new (label));
+		    widget_stack_push (stack, gtk_frame_new (label));
 		    break;
 		}
 	    case 'L':{
@@ -510,7 +530,7 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 		    next_arg (GTK_CAULDRON_TYPE_CHAR_P, user_data, &label);
 		    w = gtk_label_new (label);
 		    user_callbacks (w, p + 1, next_arg, user_data, accel_table);
-		    gtk_cauldron_box_add (widget_stack_top (), w, &p, pixels_per_space);
+		    gtk_cauldron_box_add (widget_stack_top (stack), w, &p, pixels_per_space);
 		    gtk_widget_show (w);
 		    break;
 		}
@@ -551,13 +571,13 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 		    }
 		    user_callbacks (w, p + 1, next_arg, user_data, accel_table);
 		    if (table) {
-			gtk_cauldron_box_add (widget_stack_top (), table, &p, pixels_per_space);
+			gtk_cauldron_box_add (widget_stack_top (stack), table, &p, pixels_per_space);
 			gtk_table_attach (GTK_TABLE (table), w, 0, 1, 0, 1,
 				      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			       GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 			gtk_widget_show (table);
 		    } else {
-			gtk_cauldron_box_add (widget_stack_top (), w, &p, pixels_per_space);
+			gtk_cauldron_box_add (widget_stack_top (stack), w, &p, pixels_per_space);
 		    }
 		    gtk_widget_show (w);
 		    if (*result) {
@@ -619,7 +639,7 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 			w = gtk_entry_new ();
 			new_result (r, get_entry_result, w, result);
 			if (c == 'P')
-			    gtk_entry_set_visibility (GTK_ENTRY (w), TRUE);
+			    gtk_entry_set_visibility (GTK_ENTRY (w), FALSE);
 			if (*result)
 			    gtk_entry_set_text (GTK_ENTRY (w), *result);
 		    }
@@ -627,12 +647,12 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 		    w = gtk_entry_new ();
 		    new_result (r, get_entry_result, w, result);
 		    if (c == 'P')
-			gtk_entry_set_visibility (GTK_ENTRY (w), TRUE);
+			gtk_entry_set_visibility (GTK_ENTRY (w), FALSE);
 		    if (*result)
 			gtk_entry_set_text (GTK_ENTRY (w), *result);
 #endif
 		    user_callbacks (w, p + 1, next_arg, user_data, accel_table);
-		    gtk_cauldron_box_add (widget_stack_top (), w, &p, pixels_per_space);
+		    gtk_cauldron_box_add (widget_stack_top (stack), w, &p, pixels_per_space);
 		    gtk_widget_show (w);
 		    break;
 		}
@@ -649,7 +669,7 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 		    } else
 			w = gtk_button_new ();
 		    b[ncauldron_button].label = label;
-		    b[ncauldron_button].result = &result;
+		    b[ncauldron_button].result = &return_result;
 		    if (option_is_present (p + 1, 'r'))
 			b[ncauldron_button].r = r;
 		    else
@@ -662,7 +682,7 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 			      GTK_SIGNAL_FUNC (gtk_cauldron_button_exit),
 					(gpointer) & b[ncauldron_button]);
 		    user_callbacks (w, p + 1, next_arg, user_data, accel_table);
-		    gtk_cauldron_box_add (widget_stack_top (), w, &p, pixels_per_space);
+		    gtk_cauldron_box_add (widget_stack_top (stack), w, &p, pixels_per_space);
 		    gtk_widget_show (w);
 		    ncauldron_button++;
 		    break;
@@ -679,7 +699,7 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 		    GTK_TOGGLE_BUTTON (w)->active = *result;
 		    new_result (r, get_check_result, w, result);
 		    user_callbacks (w, p + 1, next_arg, user_data, accel_table);
-		    gtk_cauldron_box_add (widget_stack_top (), w, &p, pixels_per_space);
+		    gtk_cauldron_box_add (widget_stack_top (stack), w, &p, pixels_per_space);
 		    gtk_widget_show (w);
 		    break;
 		}
@@ -708,16 +728,16 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 			w = gtk_spin_button_new (0, climb_rate, digits);
 		    }
 		    user_callbacks (w, p + 1, next_arg, user_data, accel_table);
-		    gtk_cauldron_box_add (widget_stack_top (), w, &p, pixels_per_space);
+		    gtk_cauldron_box_add (widget_stack_top (stack), w, &p, pixels_per_space);
 		    gtk_widget_show (w);
 		} else {
-		    if (GTK_IS_HBOX (widget_stack_top ())) {
+		    if (GTK_IS_HBOX (widget_stack_top (stack))) {
 			w = gtk_vseparator_new ();
 		    } else {
 			w = gtk_hseparator_new ();
 		    }
 		    user_callbacks (w, p + 1, next_arg, user_data, accel_table);
-		    gtk_cauldron_box_add (widget_stack_top (), w, &p, pixels_per_space);
+		    gtk_cauldron_box_add (widget_stack_top (stack), w, &p, pixels_per_space);
 		    gtk_widget_show (w);
 		}
 		break;
@@ -734,7 +754,7 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 		    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w), *result);
 		    new_result (r, get_check_result, w, result);
 		    user_callbacks (w, p + 1, next_arg, user_data, accel_table);
-		    gtk_cauldron_box_add (widget_stack_top (), w, &p, pixels_per_space);
+		    gtk_cauldron_box_add (widget_stack_top (stack), w, &p, pixels_per_space);
 		    gtk_widget_show (w);
 		    break;
 		}
@@ -745,7 +765,7 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 		    next_arg (GTK_CAULDRON_TYPE_USERDATA_P, user_data, &data);
 		    w = (*func) (window, data);		/* `window' is just to fill the first arg and shouldn't be needed by the user */
 		    if (w) {
-			gtk_cauldron_box_add (widget_stack_top (), w, &p, pixels_per_space);
+			gtk_cauldron_box_add (widget_stack_top (stack), w, &p, pixels_per_space);
 			gtk_widget_show (w);
 		    } else {
 			g_warning ("user callback GtkCauldronCustomCallback returned null");
@@ -758,9 +778,45 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 	case ')':
 	case ']':
 	    radio_group = 0;
-	    w = widget_stack_pop ();
-	    user_callbacks (w, p + 1, next_arg, user_data, accel_table);
-	    gtk_cauldron_box_add (widget_stack_top (), w, &p, pixels_per_space);
+	    w = widget_stack_pop (stack);
+	    if (GTK_IS_NOTEBOOK(w)) {
+		gtk_notebook_set_page (GTK_NOTEBOOK (w), 0);
+		w = widget_stack_pop (stack);
+	    }
+	    if (option_is_present (p + 1, 'n')) {
+		gchar *label;
+/* adding to a notebook requires special handling : */
+		if (!GTK_IS_NOTEBOOK(widget_stack_top (stack))) {
+		    GtkWidget *notebook;
+		    notebook = gtk_notebook_new ();
+		    if (option_is_present (p + 1, 'v'))
+			gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_LEFT);
+		    while (p[1] && strchr (CAULDRON_FMT_OPTIONS, p[1]))
+			p++;
+		    gtk_cauldron_box_add (widget_stack_top (stack), notebook, &p, pixels_per_space);
+		    gtk_widget_show (notebook);
+		    widget_stack_push (stack, notebook);
+		} else {
+		    while (p[1] && strchr (CAULDRON_FMT_OPTIONS, p[1]))
+			p++;
+		}
+		next_arg (GTK_CAULDRON_TYPE_CHAR_P, user_data, &label);
+		user_callbacks (w, p + 1, next_arg, user_data, accel_table);
+		gtk_notebook_append_page (GTK_NOTEBOOK (widget_stack_top (stack)), w, gtk_label_new (label));
+	    } else if (option_is_present (p + 1, 'v') || option_is_present (p + 1, 'h')) {
+		GtkWidget *scrolled_window;
+		scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+						option_is_present (p + 1, 'h') ? GTK_POLICY_ALWAYS : GTK_POLICY_AUTOMATIC,
+						option_is_present (p + 1, 'v') ? GTK_POLICY_ALWAYS : GTK_POLICY_AUTOMATIC);
+		gtk_container_add (GTK_CONTAINER (scrolled_window), w);
+		gtk_widget_show (scrolled_window);
+		user_callbacks (scrolled_window, p + 1, next_arg, user_data, accel_table);
+		gtk_cauldron_box_add (widget_stack_top (stack), scrolled_window, &p, pixels_per_space);
+	    } else {
+		user_callbacks (w, p + 1, next_arg, user_data, accel_table);
+		gtk_cauldron_box_add (widget_stack_top (stack), w, &p, pixels_per_space);
+	    }
 	    gtk_widget_show (w);
 	    break;
 	default:{
@@ -774,24 +830,26 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 	    }
 	}
     }
-    if (widget_stack_pointer != 1) {
+    if (stack->widget_stack_pointer != 1) {
 	g_error ("bracketing error in call to gtk_dialog_cauldron()");
 	return 0;
     }
-    w = widget_stack_pop ();
+    w = widget_stack_pop (stack);
     gtk_widget_show (w);
 
     while (ntext_widget-- > 0) {
 	gtk_text_freeze (GTK_TEXT (text_widget[ntext_widget]));
 	gtk_text_insert (GTK_TEXT (text_widget[ntext_widget]), NULL, NULL, NULL, text[ntext_widget], -1);
 	gtk_text_thaw (GTK_TEXT (text_widget[ntext_widget]));
-	ntext_widget--;
     }
 
+    if (options & GTK_CAULDRON_GRAB)
+	gtk_grab_add (window);
     gtk_main ();
 
     gtk_window_remove_accel_group (GTK_WINDOW (window), accel_table);
-
+    gtk_object_destroy (GTK_OBJECT (window));
+    
     if (r) {
 	while (r->prev)
 	    r = r->prev;
@@ -802,7 +860,9 @@ gchar *gtk_dialog_cauldron_parse (gchar * title, glong options, const gchar * fo
 	    r = next;
 	}
     }
-    return result;
+    free (fmt);
+    free (stack);
+    return return_result;
 }
 
 static void next_arg (gint type, va_list * ap, void *result)
