@@ -41,8 +41,6 @@ static gint gtk_dial_expose                   (GtkWidget         *widget,
 					       GdkEventExpose    *event);
 static void gtk_dial_make_pixmap              (GtkDial           *dial);
 static void gtk_dial_paint                    (GtkDial           *dial);
-static gint gtk_dial_configure_event          (GtkWidget         *widget,
-					       GdkEventConfigure *event); 
 static gint gtk_dial_button_press             (GtkWidget         *widget,
 					       GdkEventButton    *event);
 static gint gtk_dial_button_release           (GtkWidget         *widget,
@@ -105,7 +103,6 @@ gtk_dial_class_init (GtkDialClass *class)
   widget_class->size_request = gtk_dial_size_request;
   widget_class->size_allocate = gtk_dial_size_allocate;
   /* Widget events */
-  widget_class->configure_event = gtk_dial_configure_event;
   widget_class->button_press_event = gtk_dial_button_press;
   widget_class->button_release_event = gtk_dial_button_release;
   widget_class->motion_notify_event = gtk_dial_motion_notify;
@@ -116,6 +113,7 @@ gtk_dial_init (GtkDial *dial)
 {
   dial->button           = 0;
   dial->policy           = GTK_UPDATE_CONTINUOUS;
+  dial->view_only        = FALSE;
   dial->timer            = 0;
   dial->radius           = 0;
   dial->pointer_width    = 0;
@@ -169,6 +167,15 @@ gtk_dial_get_adjustment (GtkDial *dial)
   return dial->adjustment;
 }
 
+void gtk_dial_set_view_only (GtkDial *dial,
+			     gboolean view_only)
+{
+  g_return_if_fail(dial != NULL);
+  g_return_if_fail(GTK_IS_DIAL (dial));
+
+  dial->view_only = view_only;
+}
+
 void
 gtk_dial_set_update_policy (GtkDial      *dial,
 			     GtkUpdateType  policy)
@@ -214,8 +221,8 @@ gfloat
 gtk_dial_set_percentage (GtkDial *dial, gfloat percent)
 {
  
-  g_return_if_fail (dial != NULL);
-  g_return_if_fail (GTK_IS_DIAL (dial));
+  g_return_val_if_fail (dial != NULL, 0.0);
+  g_return_val_if_fail (GTK_IS_DIAL (dial), 0.0);
 
   if (percent < 1.0)
     {
@@ -230,8 +237,8 @@ gtk_dial_set_percentage (GtkDial *dial, gfloat percent)
 gfloat
 gtk_dial_get_percentage (GtkDial *dial)
 {
-  g_return_if_fail (dial != NULL);
-  g_return_if_fail (GTK_IS_DIAL (dial));
+  g_return_val_if_fail (dial != NULL, 0.0);
+  g_return_val_if_fail (GTK_IS_DIAL (dial), 0.0);
 
   return dial->percentage;
 }
@@ -239,8 +246,8 @@ gtk_dial_get_percentage (GtkDial *dial)
 gfloat
 gtk_dial_get_value (GtkDial *dial)
 {
-  g_return_if_fail (dial != NULL);
-  g_return_if_fail (GTK_IS_DIAL (dial));
+  g_return_val_if_fail (dial != NULL, 0.0);
+  g_return_val_if_fail (GTK_IS_DIAL (dial), 0.0);
 
   return dial->adjustment->value;
 }
@@ -248,8 +255,8 @@ gtk_dial_get_value (GtkDial *dial)
 gfloat
 gtk_dial_set_value (GtkDial *dial, gfloat value)
 {
-  g_return_if_fail (dial != NULL);
-  g_return_if_fail (GTK_IS_DIAL (dial));
+  g_return_val_if_fail (dial != NULL, 0.0);
+  g_return_val_if_fail (GTK_IS_DIAL (dial), 0.0);
 
   if (value < dial->adjustment->upper)
     {
@@ -327,7 +334,6 @@ gtk_dial_size_allocate (GtkWidget     *widget,
 			GtkAllocation *allocation)
 {
   GtkDial *dial;
-  int     r_width, r_height, kludge;
 
   g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_DIAL (widget));
@@ -349,12 +355,6 @@ gtk_dial_size_allocate (GtkWidget     *widget,
 
 }
 
-static gint
-gtk_dial_configure_event (GtkWidget *widget, GdkEventConfigure *event)
-{
-  g_print ("Configure event called.\n");
-}
-
 static gint
 gtk_dial_expose (GtkWidget      *widget,
 		 GdkEventExpose *event)
@@ -519,13 +519,15 @@ gtk_dial_timer (GtkDial *dial)
 static void
 gtk_dial_update_mouse (GtkDial *dial, gint x, gint y)
 {
-  GtkWidget *widget;
   gint xc, yc;
   gfloat old_value;
 
   g_return_if_fail (dial != NULL);
   g_return_if_fail (GTK_IS_DIAL (dial));
 
+  if(dial->view_only)
+    return;
+
   xc = GTK_WIDGET(dial)->allocation.width / 2;
   yc = GTK_WIDGET(dial)->allocation.height / 2;
 
