@@ -11,7 +11,6 @@
 #include "libgnomeui/gnome-app-helper.h"
 #include "libgnomeui/gnome-app-util.h"
 #include "libgnomeui/gnome-actionarea.h"
-#ifdef GTK_HAVE_FEATURES_1_1_0
 #include "libgnomeui/gnome-canvas.h"
 #include "libgnomeui/gnome-canvas-image.h"
 #include "libgnomeui/gnome-canvas-line.h"
@@ -19,7 +18,7 @@
 #include "libgnomeui/gnome-canvas-text.h"
 #include "libgnomeui/gnome-canvas-util.h"
 #include "libgnomeui/gnome-canvas-widget.h"
-#endif
+#include "libgnomeui/gnome-color-picker.h"
 #include "libgnomeui/gnome-color-selector.h"
 #include "libgnomeui/gnome-dentry-edit.h"
 #include "libgnomeui/gnome-dialog.h"
@@ -29,15 +28,14 @@
 #include "libgnomeui/gnome-file-entry.h"
 #include "libgnomeui/gnome-font-selector.h"
 #include "libgnomeui/gnome-geometry.h"
+#include "libgnomeui/gnome-gtk-utils.h"
 #include "libgnomeui/gnome-icon-list.h"
 #include "libgnomeui/gnome-icon-sel.h"
 #include "libgnomeui/gnome-less.h"
 #include "libgnomeui/gnome-messagebox.h"
 #include "libgnomeui/gnome-net.h"
 #include "libgnomeui/gnome-number-entry.h"
-#ifdef GTK_HAVE_FEATURES_1_1_0
-#include "libgnomeui/gnome-popupmenu.h"
-#endif
+#include "libgnomeui/gnome-popup-menu.h"
 #include "libgnomeui/gnome-pixmap.h"
 #include "libgnomeui/gnome-preferences.h"
 #include "libgnomeui/gnome-propertybox.h"
@@ -63,12 +61,17 @@
 #include "libgnomeui/gnome-mdi.h"
 #include "libgnomeui/gnome-mdi-child.h"
 #include "libgnomeui/gnometypebuiltins.h"
+#include "libgnomeui/gnome-winhints.h"
 
 BEGIN_GNOME_DECLS
 
 error_t gnome_init (char *app_id, struct argp *app_parser,
 		    int argc, char **argv,
 		    unsigned int flags, int *arg_index);
+error_t gnome_init_with_data (char *app_id, struct argp *app_parser,
+			      int argc, char **argv,
+			      unsigned int flags, int *arg_index,
+			      void *user_data);
 
 END_GNOME_DECLS
 
