@@ -98,19 +98,12 @@ create_menu(GtkWidget *window)
 {
 	void gnome_stock_menu_accel_dlg(char *);
 	GtkWidget *menubar, *w, *menu;
-#ifdef GTK_HAVE_FEATURES_1_1_0
 	GtkAccelGroup *accel;
-#else
-	GtkAcceleratorTable *accel;
-#endif
 	int i = 0;
 	guchar key;
 	guint8 mod;
-#ifdef GTK_HAVE_FEATURES_1_1_0
+
 	accel = gtk_accel_group_new();
-#else
-	accel = gtk_accelerator_table_new();
-#endif
 	menubar = gtk_menu_bar_new();
 	gtk_widget_show(menubar);
 
@@ -119,55 +112,40 @@ create_menu(GtkWidget *window)
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_NEW, _("New..."));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_NEW, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_OPEN, _("Open..."));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_OPEN, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_SAVE, _("Save"));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_SAVE, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_SAVE_AS, _("Save as..."));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_SAVE_AS, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_CLOSE, _("Close"));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_CLOSE, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
@@ -178,11 +156,8 @@ create_menu(GtkWidget *window)
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_PRINT, _("Print..."));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_PRINT, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
@@ -198,11 +173,8 @@ create_menu(GtkWidget *window)
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_QUIT, _("Quit"));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_QUIT, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_signal_connect_object(GTK_OBJECT(w), "activate",
 				  (GtkSignalFunc)message_dlg,
 				  GTK_OBJECT(window));
@@ -219,11 +191,8 @@ create_menu(GtkWidget *window)
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_UNDO, _("Undo"));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_UNDO, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
@@ -244,33 +213,24 @@ create_menu(GtkWidget *window)
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_CUT, _("Cut"));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_CUT, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_COPY, _("Copy"));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_COPY, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_PASTE, _("Paste"));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_PASTE, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
@@ -281,11 +241,8 @@ create_menu(GtkWidget *window)
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_PROP, _("Properties..."));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_PROP, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_signal_connect(GTK_OBJECT(w), "activate",
 			   GTK_SIGNAL_FUNC(prop_dlg), NULL);
 	gtk_menu_append(GTK_MENU(menu), w);
@@ -294,11 +251,8 @@ create_menu(GtkWidget *window)
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_PREF, _("Preferences..."));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_PREF, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_signal_connect_object(GTK_OBJECT(w), "activate",
 				  GTK_SIGNAL_FUNC(gnome_stock_menu_accel_dlg),
 				  NULL);
@@ -312,11 +266,8 @@ create_menu(GtkWidget *window)
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_SCORES, _("Scores..."));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_SCORES, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
@@ -331,11 +282,8 @@ create_menu(GtkWidget *window)
 	w = gnome_stock_menu_item(GNOME_STOCK_MENU_ABOUT, _("About"));
 	gtk_widget_show(w);
 	if (gnome_stock_menu_accel(GNOME_STOCK_MENU_ABOUT, &key, &mod))
-#ifdef GTK_HAVE_FEATURES_1_1_0
 		gtk_widget_add_accelerator(w, "activate",  accel, key, mod, 0);
-#else
-		gtk_widget_install_accelerator(w, accel, "activate", key, mod);
-#endif
+
 	gtk_menu_append(GTK_MENU(menu), w);
 	menu_items[i++] = w;
 
@@ -347,11 +295,8 @@ create_menu(GtkWidget *window)
 
 	menu_items[i] = NULL;
 	/* g_print("%d menu items\n", i); */
-#ifdef GTK_HAVE_FEATURES_1_1_0
 	gtk_window_add_accel_group(GTK_WINDOW(window), accel);
-#else
-	gtk_window_add_accelerator_table (GTK_WINDOW (window), accel);
-#endif
+
 	return menubar;
 }
 
@@ -774,6 +719,28 @@ fill_table(GtkWidget *window, GtkTable *table)
 	gtk_widget_show(w);
 	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
 
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_CONVERT));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_CONVERT));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Convert");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_JUMP_TO));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_JUMP_TO));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Jump To");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
 	column = 0; row += 3;
 	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_BOOK_RED));
 	gtk_widget_show(w);
