@@ -5,9 +5,8 @@
 
 GtkWidget *create_newwin(gboolean normal, gchar *appname, gchar *title);
 
-#ifdef GTK_HAVE_FEATURES_1_1_0
 void create_canvas (void);
 GtkWidget *create_canvas_primitives (void);
 GtkWidget *create_canvas_arrowhead (void);
 GtkWidget *create_canvas_fifteen (void);
-#endif
+GtkWidget *create_canvas_features (void);
