@@ -58,7 +58,7 @@ void vt_delete_chars(struct vt_em *vt, int count);
 void vt_insert_lines(struct vt_em *vt, int count);
 void vt_delete_lines(struct vt_em *vt, int count);
 void vt_clear_lines(struct vt_em *vt, int top, int count);
-void vt_clear_eol(struct vt_em *vt);
+void vt_clear_line_portion(struct vt_em *vt, int start_col, int end_col);
 
 static unsigned char vt_remap_dec[256];
 
@@ -379,7 +379,8 @@ void vt_delete_lines(struct vt_em *vt, int count)
     
     /* clear it */
     for (i=0;i<wn->width;i++) {
-      if ((wn->data[i]&0xff) && ((wn->data[i] & 0xff) != ' ')) {
+      if ((wn->data[i]&0xff) && (((wn->data[i] & 0xff) != ' ') || 
+		((wn->data[i] & (0x7ffff)) != vt->attr))) {
 	wn->data[i] = vt->attr|VTATTR_CHANGED; /* FIXME: include attributes */
       }
     }
@@ -416,15 +417,15 @@ void vt_clear_lines(struct vt_em *vt, int top, int count)
   }
 }
 
-void vt_clear_eol(struct vt_em *vt)
+void vt_clear_line_portion(struct vt_em *vt, int start_col, int end_col)
 {
   struct vt_line *this;
   int i;
 
-  d(printf("vt_clear_eol()\n"));
+  d(printf("vt_clear_line_portion()\n"));
 
   this = vt->this;
-  for(i=vt->cursorx;i<this->width;i++) {
+  for(i=start_col;i<end_col;i++) {
     this->data[i] = vt->attr | VTATTR_CHANGED;
   }
   this->modcount+=(this->width-vt->cursorx);
@@ -577,16 +578,25 @@ static void vt_delete_line(struct vt_em *vt)
 static void vt_cleareos(struct vt_em *vt)
 {
   d(printf("clear screen/end of screen\n"));
-  vt_clear_eol(vt);
+  vt_clear_line_portion(vt, vt->cursorx, vt->this->width);
   if (vt->cursory<vt->height) {
     vt_clear_lines(vt, vt->cursory+1, vt->height);
   }
 }
 
-static void vt_cleareol(struct vt_em *vt)
+static void vt_clear_lineportion(struct vt_em *vt)
 {
-  d(printf("Clear end of line\n"));
-  vt_clear_eol(vt);
+  d(printf("Clear part of line\n"));
+  if (vt->argcnt>1) {
+    /* eat the command */
+  } else if (vt->argcnt==0 || *vt->args[0] == '0') {
+    vt_clear_line_portion(vt, vt->cursorx, vt->this->width);
+  } else if (*vt->args[0] == '1') {
+    vt_clear_line_portion(vt, 0, vt->cursorx + 1);
+  } else if (*vt->args[0] == '2') {
+    vt_clear_line_portion(vt, 0, vt->this->width);
+  }
+  /* eat bad parameters */
 }
 
 
@@ -694,6 +704,10 @@ static void vt_goto(struct vt_em *vt)
   } else {
     d(printf("position had too many parameters\n"));
   }
+  if (vt->cursorx < 0)
+    vt->cursorx = 0;
+  if (vt->cursory < 0)
+    vt->cursory = 0;
   if (vt->cursorx >= vt->width)
     vt->cursorx = vt->width-1;
   if (vt->cursory >= vt->height)
@@ -959,7 +973,7 @@ struct vt_jump vtjumps[] = {
   {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT|VT_ARG},	/* <=>? */
   {vt_insert_char,VT_EBL}, {vt_up,VT_BOL}, {vt_down,VT_BOL}, {vt_right,VT_BOL},	/* @ABC */
   {vt_left,VT_BOL}, {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT},	/* DEFG */
-  {vt_goto,VT_EBL}, {0,VT_LIT}, {vt_cleareos,VT_EBL}, {vt_cleareol,VT_EBL},	/* HIJK */
+  {vt_goto,VT_EBL}, {0,VT_LIT}, {vt_cleareos,VT_EBL}, {vt_clear_lineportion,VT_EBL},	/* HIJK */
   {vt_insert_line,VT_EBL}, {vt_delete_line,VT_EBL|VT_ESC}, {0,VT_LIT}, {0,VT_LIT},	/* LMNO */
   {vt_delete_char,VT_EBL}, {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT},	/* PQRS */
   {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT}, {0,VT_LIT},	/* TUVW */
@@ -989,6 +1003,7 @@ void parse_vt(struct vt_em *vt, char *ptr, int length)
   register int c;
   register int state;
   register int mode;
+  int handled;
   struct vt_jump *modes = vtjumps;
   char *ptrend;
   void (*process)(struct vt_em *vt);	/* process function */
@@ -999,111 +1014,118 @@ void parse_vt(struct vt_em *vt, char *ptr, int length)
     c=(*ptr++) & 0xff;		/* convert to unsigned byte */
     mode = modes[c & 0x7f].modes;
     process = modes[c & 0x7f].process;
-    switch(state) {
-    case 0:
-      if (mode & VT_LIT) {
-	/* remap character? */
-	if (vt->remaptable)
-	  c=vt->remaptable[c];
+    do {
+      handled = 1;
+      switch(state) {
+      case 0:
+	if (mode & VT_LIT) {
+	  /* remap character? */
+	  if (vt->remaptable)
+	    c=vt->remaptable[c];
 
-	/* insert mode? */
-	if (vt->mode & VTMODE_INSERT)
-	  vt_insert_chars(vt, 1);
+	  /* insert mode? */
+	  if (vt->mode & VTMODE_INSERT)
+	    vt_insert_chars(vt, 1);
 
-	/* need to wrap? */
-	if (vt->cursorx>=vt->width) {
-	  vt_lf(vt);
-	  vt->cursorx=0;
+	  /* need to wrap? */
+	  if (vt->cursorx>=vt->width) {
+	    vt_lf(vt);
+	    vt->cursorx=0;
+	  }
+
+	  /* output character */
+	  vt->this->data[vt->cursorx] = ((vt->attr | VTATTR_CHANGED) & 0xffff0000) | c;
+	  vt->this->modcount++;
+	  d(printf("literal %c\n", c));
+	  vt->cursorx++;
+	} else if (mode & VT_CON) {
+	  process(vt);
+	} else if (c==27) {
+	  state=1;
 	}
-
-	/* output character */
-	vt->this->data[vt->cursorx] = ((vt->attr | VTATTR_CHANGED) & 0xffff0000) | c;
-	vt->this->modcount++;
-	d(printf("literal %c\n", c));
-	vt->cursorx++;
-      } else if (mode & VT_CON) {
-	process(vt);
-      } else if (c==27) {
-	state=1;
-      }
-      /* else ignore */
-      break;
-    case 1:			/* received 'esc', next byte */
-      if (mode & VT_ESC) {	/* got a \Ex sequence */
-	vt->argcnt = 0;
-	process(vt);
-	state=0;
-      } else if (c=='[') {
-	vt->argptr = vt->args;	/* initialise output arg pointers */
-	vt->outptr = vt->argptr[0];
-	vt->outend = vt->outptr+VTPARAM_ARGMAX;
-	state = 2;
-      } else if (c=='O') {
-	state = 3;
-      } else if (c==']') {	/* set text parameters, read parameters */
-	state = 4;
-	vt->argptr = vt->args;
-	vt->outptr = vt->argptr[0];
-	vt->outend = vt->outptr+VTPARAM_ARGMAX*VTPARAM_MAXARGS-1;
-      } else if (mode & VT_EXA) {
-	vt->args[0][0]=c;
-	state = 5;
-      } else {
-	state = 0;		/* dont understand input */
-      }
-      break;
-    case 2:
-      if (mode & VT_ARG) {
-	if (vt->outptr<vt->outend) /* truncate excessive args */
-	  *(vt->outptr)++=c;
-      } else if (c==';') {	/* looking for 'arg;arg;...' */
-	if (vt->argcnt<VTPARAM_MAXARGS) { /* goto next argument */
-	  *(vt->outptr)=0;
-	  vt->argptr++;
+	/* else ignore */
+	break;
+      case 1:			/* received 'esc', next byte */
+	if (mode & VT_ESC) {	/* got a \Ex sequence */
+	  vt->argcnt = 0;
+	  process(vt);
+	  state=0;
+	} else if (c=='[') {
+	  vt->argptr = vt->args;	/* initialise output arg pointers */
 	  vt->outptr = vt->argptr[0];
 	  vt->outend = vt->outptr+VTPARAM_ARGMAX;
+	  state = 2;
+	} else if (c=='O') {
+	  state = 3;
+	} else if (c==']') {	/* set text parameters, read parameters */
+	  state = 4;
+	  vt->argptr = vt->args;
+	  vt->outptr = vt->argptr[0];
+	  vt->outend = vt->outptr+VTPARAM_ARGMAX*VTPARAM_MAXARGS-1;
+	} else if (mode & VT_EXA) {
+	  vt->args[0][0]=c;
+	  state = 5;
+	} else {
+	  state = 0;		/* dont understand input */
 	}
-	/* others are ignored */
-      } else if (mode & VT_EXB) {
-	if (vt->outptr!=vt->argptr[0])
-	  vt->argptr++;
-	*(vt->outptr)=0;
-	vt->argcnt = (vt->argptr-vt->args);
-	process(vt);
+	break;
+      case 2:
+	if (mode & VT_ARG) {
+	  if (vt->outptr<vt->outend) /* truncate excessive args */
+	    *(vt->outptr)++=c;
+	} else if (c==';' || c==':') {	/* looking for 'arg;arg;...' */
+	  if (vt->argcnt<VTPARAM_MAXARGS) { /* goto next argument */
+	    *(vt->outptr)=0;
+	    vt->argptr++;
+	    vt->outptr = vt->argptr[0];
+	    vt->outend = vt->outptr+VTPARAM_ARGMAX;
+	  }
+	  /* others are ignored */
+	} else if (mode & VT_EXB) {
+	  if (vt->outptr!=(char *) vt->argptr[0])
+	    vt->argptr++;
+	  *(vt->outptr)=0;
+	  vt->argcnt = (vt->argptr-vt->args);
+	  process(vt);
+	  state=0;
+	} else {
+	  d(printf("unknown option '%c'\n", c));
+	  state=0;		/* unexpected escape sequence - ignore */
+	}
+	break;
+      case 3:			/* \EOx */
+	if (mode & VT_EXO) {
+	  process(vt);
+	}	/* ignore otherwise */
 	state=0;
-      } else {
-	(printf("unknown option '%c'\n", c));
-	state=0;		/* unexpected escape sequence - ignore */
+	break;
+      case 4:			/* \E]..;...BEL */
+	if (c==0x07) {
+	  /* handle output */
+	  *(vt->outptr)=0;
+	  d(printf("received text mode: %s\n", vt->argptr[0]));
+	  state = 0;
+	} else if (c==0x0a) {
+	  state = 0;		/* abort command */
+	} else {
+	  if (vt->outptr<vt->outend) /* truncate excessive args */
+	    *(vt->outptr)++=c;
+	}
+	break;
+      case 5:			/* \E?x */
+	vt->args[0][1]=c;
+	if (vt->args[0][0] < 0x7f) {
+	  process = modes[vt->args[0][0]].process;
+	  if (process) {		/* check, if doesn't exist, then ignore and lose */
+	    process(vt);
+	  }
+	} else {
+	  handled = 0;
+	}
+	state=0;
+	break;
       }
-      break;
-    case 3:			/* \EOx */
-      if (mode & VT_EXO) {
-	process(vt);
-      }	/* ignore otherwise */
-      state=0;
-      break;
-    case 4:			/* \E]..;...BEL */
-      if (c==0x07) {
-	/* handle output */
-	*(vt->outptr)=0;
-	printf("received text mode: %s\n", vt->argptr[0]);
-	state = 0;
-      } else if (c==0x0a) {
-	state = 0;		/* abort command */
-      } else {
-	if (vt->outptr<vt->outend) /* truncate excessive args */
-	  *(vt->outptr)++=c;
-      }
-      break;
-    case 5:			/* \E?x */
-      vt->args[0][1]=c;
-      process = modes[vt->args[0][0]].process;
-      if (process) {		/* check, if doesn't exist, then ignore and lose */
-	process(vt);
-      }
-      state=0;
-      break;
-    }
+    } while (!handled);
   }
   vt->state = state;
 }
@@ -1203,19 +1225,14 @@ struct vt_em *vt_init(struct vt_em *vt, int width, int height)
 /*
   start a child process running in the VT emulator
 
-   fork the child process.
-
-     NOTE: Uses the glibc2+ function 'forkpty()' to take care of
-      1. finding a free pty
-      2. forking a child on that pty
-      3. setting the child's master terminal to be that pty
+   fork the child process
 
 */
 int vt_forkpty(struct vt_em *vt)
 {
   char ttyname[256];
 
-  vt->childpid = init_subshell(&vt->childfd, ttyname);
+  vt->childpid = init_subshell(&vt->childfd, ttyname, &vt->msgfd);
   if (vt->childpid>0) {
     fcntl(vt->childfd, F_SETFL, O_NONBLOCK);
     resize_subshell(vt->childfd, vt->width, vt->height, vt->width*8, vt->height*8); /* FIXME: approx sizes only */
@@ -1263,7 +1280,8 @@ int vt_closepty(struct vt_em *vt)
 
   if (vt->childfd != -1){
 	  ret = close(vt->childfd);
-	  vt->childfd = -1;
+	  close_msgfd(vt->childpid);
+	  vt->msgfd = vt->childfd = -1;
   } else
 	  ret = 0;
   return ret;
