@@ -72,6 +72,7 @@ struct vt_em {
 
   int childpid;			/* child process id */
   int childfd;			/* child file descriptor (for read/write) */
+  int msgfd;			/* "it's dead" messages come through here */
 
   int savex,savey;		/* saved cursor position */
   struct vt_line *savethis;
@@ -89,9 +90,9 @@ struct vt_em {
 
   uint32 mode;			/* vt modes.  see below */
 
-  char *args[VTPARAM_MAXARGS];	/* run-time emulation arguments */
+  unsigned char *args[VTPARAM_MAXARGS];	/* run-time emulation arguments */
   char args_mem[VTPARAM_MAXARGS*VTPARAM_ARGMAX];
-  char **argptr;
+  unsigned char **argptr;
   char *outptr;
   char *outend;
   int argcnt;
