@@ -61,6 +61,7 @@ static void zvt_term_child_died(ZvtTerm *term);
 static gint zvt_term_cursor_blink(gpointer data);
 static void zvt_term_scrollbar_moved (GtkAdjustment *adj, GtkWidget *widget);
 static void zvt_term_readdata(gpointer data, gint fd, GdkInputCondition condition);
+static void zvt_term_readmsg(gpointer data, gint fd, GdkInputCondition condition);
 
 static void zvt_term_fix_scrollbar(ZvtTerm *term);
 
@@ -153,12 +154,15 @@ zvt_term_init (ZvtTerm *term)
   term->cursor_on = 0;
   term->cursor_filled = 1;
   term->cursor_blink_state = 0;
+  term->scroll_on_keystroke = 0;
+  term->scroll_on_output = 0;
   term->blink_enabled = 1;
   term->ic = NULL;
   term->in_expose = 0;
 
-  /* input handler */
+  /* input handlers */
   term->input_id = -1;
+  term->msg_id = -1;
 
   zvt_term_set_font_name(term, "-misc-fixed-medium-r-semicondensed--13-120-75-75-c-60-iso8859-1");
 
@@ -193,6 +197,18 @@ zvt_term_set_blink (ZvtTerm *term, int state)
   }
 }
 
+void 
+zvt_term_set_scroll_on_keystroke(ZvtTerm *term, int state)
+{
+  term->scroll_on_keystroke = (state != 0);
+}
+
+void 
+zvt_term_set_scroll_on_output   (ZvtTerm *term, int state)
+{
+  term->scroll_on_output = (state != 0);
+}
+
 GtkWidget*
 zvt_term_new (void)
 {
@@ -344,7 +360,7 @@ zvt_term_realize (GtkWidget *widget)
 
   /* input context */
   if (gdk_im_ready() && !term->ic) {
-    GdkIMStyle style = GdkIMPreeditNothing | GdkIMStatusNothing;
+    GdkIMStyle style = GDK_IM_PREEDIT_NOTHING | GDK_IM_STATUS_NOTHING;
     /* FIXME: do we have any window yet? */
     term->ic = gdk_ic_new(widget->window, widget->window, style, NULL);
     if (!term->ic) {
@@ -463,10 +479,10 @@ zvt_term_size_allocate (GtkWidget     *widget,
 			      allocation->x, allocation->y,
 			      allocation->width, allocation->height);
 
-      /* resize the virtual terminal buffer */
+      /* resize the virtual terminal buffer, minimal size is 1x1 */
       vt_resize(&term->vx->vt,
-		allocation->width/term->charwidth,
-		allocation->height/term->charheight,
+		MAX(allocation->width/term->charwidth,1),
+		MAX(allocation->height/term->charheight,1),
 		allocation->width,
 		allocation->height);
       vt_update(term->vx, UPDATE_REFRESH|UPDATE_SCROLLBACK);/* redraw everything, unconditionally */
@@ -1067,9 +1083,8 @@ int zvt_term_forkpty(ZvtTerm *term)
 
   pid = vt_forkpty(&term->vx->vt);
   if (pid>0) {
-    if (term->input_id==-1) {
-      term->input_id = gdk_input_add(term->vx->vt.childfd, GDK_INPUT_READ, zvt_term_readdata, term);
-    }
+    term->input_id = gdk_input_add(term->vx->vt.childfd, GDK_INPUT_READ, zvt_term_readdata, term);
+    term->msg_id = gdk_input_add(term->vx->vt.msgfd, GDK_INPUT_READ, zvt_term_readmsg, term);
   }
   return pid;
 }
@@ -1099,18 +1114,30 @@ int zvt_term_closepty(ZvtTerm *term)
     gdk_input_remove(term->input_id);
     term->input_id=-1;
   }
+  if (term->msg_id!=-1) {
+    gdk_input_remove(term->msg_id);
+    term->msg_id=-1;
+  }
   return vt_closepty(&term->vx->vt);
 }
 
 static void
 zvt_term_scroll (ZvtTerm *term, int n)
 {
-  gfloat new_value;
-  new_value = term->adjustment->value + (n * term->adjustment->page_size);
-  gtk_adjustment_set_value (term->adjustment,
-							n > 0 ? MIN(new_value, term->adjustment->upper
-										 - term->adjustment->page_size)
-							       : MAX(new_value, term->adjustment->lower));
+	gfloat new_value = 0;
+	
+	if (n)
+		new_value = term->adjustment->value + (n * term->adjustment->page_size);
+	else if (new_value == (term->adjustment->upper - term->adjustment->page_size))
+		return;
+	else
+		new_value = term->adjustment->upper - term->adjustment->page_size;
+
+	term->vx->vt.scrollbackold = -1;
+	gtk_adjustment_set_value (term->adjustment,
+				  n > 0 ? MIN(new_value, term->adjustment->upper
+					      - term->adjustment->page_size)
+				  : MAX(new_value, term->adjustment->lower));
 }
 
 /*
@@ -1290,8 +1317,10 @@ zvt_term_key_press (GtkWidget *widget, GdkEventKey *event)
       }
       d(printf ("[%s,%d,%d]\n", event->string, event->length, handled));
   }
-  if (handled && p>buffer)
+  if (handled && p>buffer) {
     vt_writechild(&vx->vt, buffer, (p-buffer));
+    if (term->scroll_on_keystroke) zvt_term_scroll (term, 0);
+  }
 
   return handled;
 }
@@ -1304,7 +1333,18 @@ static void zvt_term_child_died(ZvtTerm *term)
   g_return_if_fail (term != NULL);
   g_return_if_fail (ZVT_IS_TERM (term));
 
-  printf("Warning: Child died\n");
+  /* perhaps we should do something here? */
+}
+
+static void vtx_unrender_selection (struct _vtx *vx)
+{
+  /* need to 'un-render' selected area */
+  if (vx->selected) {		/* FIXME: modularise */
+    vx->selstartx = vx->selendx;
+    vx->selstarty = vx->selendy;
+    vt_draw_selection(vx);	/* un-render selection */
+    vx->selected = 0;
+  }
 }
 
 /*
@@ -1325,15 +1365,10 @@ static void zvt_term_readdata(gpointer data, gint fd, GdkInputCondition conditio
   ZvtTerm *term;
 
   term = (ZvtTerm *)data;
+  if (term->input_id == -1) return;
   vx = term->vx;
 
-  /* need to 'un-render' selected area */
-  if (vx->selected) {		/* FIXME: modularise */
-    vx->selstartx = vx->selendx;
-    vx->selstarty = vx->selendy;
-    vt_draw_selection(vx);	/* un-render selection */
-    vx->selected = 0;
-  }
+  vtx_unrender_selection (vx);
 
   iter=0;
   saveerrno = EAGAIN;
@@ -1361,24 +1396,71 @@ static void zvt_term_readdata(gpointer data, gint fd, GdkInputCondition conditio
   /* *always* turn the cursor back on */
   vt_cursor_state (term, 1);
   
-  /* flush all X events - this is really necessary to stop X queuing up
-     lots of screen updates and reducing interactivity on a busy terminal */
+  /* fix scroll bar */
+  if (term->scroll_on_output) zvt_term_scroll (term, 0);
+
+  /*
+   * flush all X events - this is really necessary to stop X queuing up
+   * lots of screen updates and reducing interactivity on a busy terminal
+   */
   gdk_flush();
 
-  /* read failed?
-      assume pipe cut - just quit */
-  if (count<0 && saveerrno!=EAGAIN) {
-    /* close this fd, just to make sure (removes input handler too) */
-    zvt_term_closepty(term);
+  /* read failed? oh well, that's life -- we handle dead children via
+     SIGCHLD */
 
-    /* signal application FIXME: include error/non error code */
-    gtk_signal_emit(GTK_OBJECT(term), term_signals[CHILD_DIED]);
+  zvt_term_fix_scrollbar(term);
 
-    return;
+}
+
+void
+zvt_term_feed (ZvtTerm *term, char *text, int len)
+{
+	g_return_if_fail (term != NULL);
+	g_return_if_fail (ZVT_IS_TERM (term));
+	g_return_if_fail (text != NULL);
+
+	
+	vt_cursor_state (term, 0);
+	vtx_unrender_selection (term->vx);
+	parse_vt(&term->vx->vt, text, len);
+	vt_update (term->vx, UPDATE_CHANGES);
+
+	/* *always* turn the cursor back on */
+	vt_cursor_state (term, 1);
+
+	/* fix scroll bar */
+	if (term->scroll_on_output) zvt_term_scroll (term, 0);
+
+	/*
+	 * flush all X events - this is really necessary to stop X queuing up
+	 * lots of screen updates and reducing interactivity on a busy terminal
+	 */
+	gdk_flush();
+
+	/*
+	 * read failed? oh well, that's life -- we handle dead children via
+	 *  SIGCHLD
+	 */
+
+	zvt_term_fix_scrollbar(term);
+}
+
+static void zvt_term_readmsg(gpointer data, gint fd, GdkInputCondition condition)
+{
+  ZvtTerm *term = (ZvtTerm *)data;
+
+  /* I suppose I should bother reading the message from the fd, but
+     it doesn't seem worth the trouble <shrug> */
+
+  if (term->input_id!=-1) {
+    gdk_input_remove(term->input_id);
+    term->input_id=-1;
   }
 
-  /* fix scroll bar */
-  zvt_term_fix_scrollbar(term);
+  zvt_term_closepty(term);
+
+  /* signal application FIXME: include error/non error code */
+  gtk_signal_emit(GTK_OBJECT(term), term_signals[CHILD_DIED]);
 }
 
 /*
@@ -1470,8 +1552,8 @@ void vt_draw_text(void *user_data, int col, int row, char *text, int len, int at
   }
 
   /* optimise: dont 'clear' background if in expose, and background colour == window colour */
-  if ((term->in_expose==0 && vx->back_match==0)
-      || back!=17) {
+
+  if ((term->in_expose==0) || back!=17) {
     gdk_draw_rectangle(widget->window,
 		       bgc,
 		       1,
