@@ -50,11 +50,14 @@ extern "C" {
     unsigned int cursor_blink_state:1;	/* cursor blink state */
     unsigned int blink_enabled:1;        /* Set to on if we do blinking */
     unsigned int in_expose:1;	/* updating from within expose events */
+    unsigned int scroll_on_keystroke:1;
+    unsigned int scroll_on_output:1;
 
     int charwidth;		/* size of characters */
     int charheight;
 
     gint input_id;		/* input handler id */
+    gint msg_id;		/* message handler id */
 
     /* sub-objects required */
     GtkAdjustment *adjustment;	/* scrollback position adjustement */
@@ -63,7 +66,7 @@ extern "C" {
     GdkCursor *cursor_bar,	/* I beam cursor */
       *cursor_dot,		/* the blank cursor */
       *cursor_current;		/* current active cursor */
-    guint timeout_id;		/* id of timeout function */
+    gint timeout_id;		/* id of timeout function */
     GdkFont *font,		/* current normal font */
       *font_bold;		/* current bold font */
     GdkGC *scroll_gc;		/* special GC used for scrolling */
@@ -84,6 +87,7 @@ extern "C" {
 
 
   GtkWidget*    zvt_term_new                    (void);
+  void          zvt_term_feed                   (ZvtTerm *term, char *text, int len);
   int		zvt_term_forkpty		(ZvtTerm *term);
   int           zvt_term_closepty               (ZvtTerm *term);
 
@@ -101,6 +105,8 @@ extern "C" {
   void          zvt_term_show_pointer           (ZvtTerm *term);
 
   void          zvt_term_set_blink              (ZvtTerm *term, int state);
+  void          zvt_term_set_scroll_on_keystroke(ZvtTerm *term, int state);
+  void          zvt_term_set_scroll_on_output   (ZvtTerm *term, int state);
   void          zvt_term_set_color_scheme       (ZvtTerm *term, gushort *red, gushort *grn, gushort *blu);
   void          zvt_term_set_default_color_scheme (ZvtTerm *term);
 	
